#!/usr/bin/env ruby

require 'tk'
require 'tk/canvas'

class Clock
  class Pie < TkcArc
    def set_start_time(hh,mm)
      @start_hh = hh
      @start_mm = mm
    end
    def get_start_time
      [@start_hh, @start_mm]
    end

    def set_end_time(hh,mm)
      @end_hh = hh
      @end_mm = mm
    end
    def get_end_time
      [@end_hh, @end_mm]
    end
  end

  ###################################

  def _balloon_bind(*tags)
    tags.each{|tag|
      tag.bind('Enter', proc{|x, y|
                 @cx = @c.canvasx(x)
                 @cy = @c.canvasx(y)
                 @balloon_timer.start
               }, '%x %y')
      tag.bind('Motion', proc{|x, y| 
                 @cx = @c.canvasx(x)
                 @cy = @c.canvasx(y)
                 @balloon_timer.restart
                 @balloon.state(:hidden)
                 @balloon_base.state(:hidden)
               }, '%x %y')
      tag.bind('Any-ButtonPress', proc{
                 @balloon_timer.stop
                 @balloon.state(:hidden)
                 @balloon_base.state(:hidden)
               })
      tag.bind('Leave', proc{
                 @balloon_timer.stop
                 @balloon.state(:hidden)
                 @balloon_base.state(:hidden)
               })
    }
  end
  private :_balloon_bind

  def initialize(clock24 = true)
    @clock = (clock24)? 24: 12

    @size = 200
    @cdot_size = 5

    @h_ring_size = 0.92 * @size

    @cdot_color        = 'black'
    @hour_hand_color   = 'black'
    @minute_hand_color = 'gray25'
    @second_hand_color = 'gray50'

    @mark_font     = TkFont.new('Helvetica -8')
    @mark_width    = 3
    @mark_color    = 'black'
    @hour_font     = TkFont.new('Helvetica -12 bold')
    @hour_color    = 'black'
    @submark_color = 'gray50'

    @hour_ring_color   = 'white'
    @minute_ring_color = 'gray90'

    @c = TkCanvas.new(:width=>2*@size, :height=>2*@size, 
                     :scrollregion=>[-@size, -@size, @size, @size]
                     ).pack(:fill=>:both, :expand=>true)

    @tag = TkcTag.new(@c)
    @hand_tag = TkcTag.new(@c)
    @pie_tag = TkcTag.new(@c)
    @balloon_tag = TkcTag.new(@c)

    @circle_coords = [[-0.9*@size, -0.9*@size], [0.9*@size, 0.9*@size]]
    @hour_circle_coords = [
      [-0.9*@h_ring_size, -0.9*@h_ring_size], 
      [ 0.9*@h_ring_size,  0.9*@h_ring_size]
    ]

    @oval = TkcOval.new(@c, @circle_coords, 
                        :fill=>@minute_ring_color, :tags=>[@tag])

    @balloon = TkcText.new(@c, 10000, 100000, :text=>'00:00-00:00', 
                           :tags=>[@balloon_tag], :anchor=>:nw, 
                           :fill=>'black', :font=>@hour_font)
    x1, y1, x2, y2 = @balloon.bbox
    @balloon_base = TkcRectangle.new(@c, x1-3, y1-1, x2+2, y2, 
                                     :tags=>[@balloon_tag], 
                                     :fill=>'white', :outline=>'black')
    @balloon.raise

    balloon_interval = 200
    @cx = 0
    @cy = 0
    @balloon_timer = TkAfter.new(balloon_interval, 1, proc{
                                   pie = (@c.find_overlapping(@cx, @cy, @cx, @cy) & @c.find_withtag(@pie_tag))[-1]
                                   if pie
                                     @balloon.text("#{'%02d:%02d' % pie.get_start_time}--#{'%02d:%02d' % pie.get_end_time}")
                                     @balloon.state(:normal)
                                     @balloon_base.state(:normal)
                                     @balloon.coords(@cx+12, @cy+12)
                                     x1, y1, x2, y2 = @balloon.bbox
                                     @balloon_base.coords(x1-3, y1-1, x2+2, y2)
                                   end
                                 })
    def @balloon_timer.interval(val)
      @sleep_time = val
    end
    _balloon_bind(@pie_tag, @hand_tag)

    f = TkFrame.new.pack
    TkLabel.new(f, :text=>'CURRENT:').pack(:side=>:left)
    @now = TkLabel.new(f, :text=>'00:00:00').pack(:side=>:left, :padx=>2)
    TkLabel.new(f, :text=>'  ').pack(:side=>:left)
    TkLabel.new(f, :text=>'  ').pack(:side=>:right)
    @l = TkLabel.new(f, :text=>'00:00').pack(:side=>:right, :padx=>2)
    TkLabel.new(f, :text=>'MOUSE-POINTER:').pack(:side=>:right)

    cmd = proc{|x, y|
      @l.text = '%02d:%02d' % coords_to_time(@c.canvasx(x), @c.canvasy(y))
    }
    @c.bind('Motion', cmd, '%x %y')
    @tag.bind('Motion', cmd, '%x %y')

    mark_hour = nil
    mark_min  = nil
    new_pie   = nil

    cmd_press = proc{|x, y| 
      @balloon_timer.stop
      @balloon.state(:hidden)
      @balloon_base.state(:hidden)
      unless new_pie
        mark_hour, mark_min = coords_to_time(@c.canvasx(x), @c.canvasy(y))
        new_pie = create_pie(mark_hour, mark_min, 0)
      end
    }
    cmd_motion = proc{|x, y| 
      @cx = @c.canvasx(x)
      @cy = @c.canvasx(y)
      @balloon_timer.stop
      @balloon.state(:hidden)
      @balloon_base.state(:hidden)

      if mark_hour && mark_min && new_pie
        hour, min = coords_to_time(@c.canvasx(x), @c.canvasy(y))
        set_pie_extent(new_pie, (hour*60 + min) - (mark_hour*60 + mark_min))
      end
      cmd.call(x, y)
    }
    cmd_release = proc{|x, y| 
      mark_hour = nil
      mark_min  = nil
      new_pie   = nil
    }

    @c.bind('ButtonPress-1', cmd_press, '%x %y')
    @tag.bind('ButtonPress-1', cmd_press, '%x %y')
    @c.bind('B1-Motion', cmd_motion, '%x %y')
    @tag.bind('B1-Motion', cmd_motion, '%x %y')
    @c.bind('ButtonRelease-1', cmd_release, '%x %y')
    @tag.bind('ButtonRelease-1', cmd_release, '%x %y')

    _create_hands
    _create_marks

    timer_proc = proc{
      t = Time.now
      @now.text = '%02d:%02d:%02d' % [t.hour, t.min, t.sec]
      set_hands(t.hour, t.min, t.sec)
    }

    timer_proc.call
    @timer = TkRTTimer.start(100, -1, timer_proc)
  end

  def _create_marks
    @mark_tag = TkcTag.new(@c)

    size = @h_ring_size
    h_ratio = 0.75

    TkcOval.new(@c, -0.9*size, -0.9*size, 0.9*size, 0.9*size, 
                :fill=>@hour_ring_color, :outline=>'')
    TkcOval.new(@c, -0.9*size, -0.9*size, 0.9*size, 0.9*size, 
                :tags=>[@tag, @mark_tag], :fill=>'', :outline=>@mark_color)

    TkcLine.new(@c, 0, -0.90*size, 0, -0.85*size, 
                :tags=>[@tag, @mark_tag], 
                :width=>@mark_width, :fill=>@mark_color)
    TkcLine.new(@c, 0.90*size, 0, 0.85*size, 0, 
                :tags=>[@tag, @mark_tag], 
                :width=>@mark_width, :fill=>@mark_color)
    TkcLine.new(@c, 0, 0.90*size, 0, 0.85*size, 
                :tags=>[@tag, @mark_tag], 
                :width=>@mark_width, :fill=>@mark_color)
    TkcLine.new(@c, -0.90*size, 0, -0.85*size, 0, 
                :tags=>[@tag, @mark_tag], 
                :width=>@mark_width, :fill=>@mark_color)

    TkcText.new(@c, [0, -h_ratio*size], :text=>0, 
                :tags=>[@tag, @mark_tag], :font=>@hour_font, 
                :anchor=>'s', :fill=>@hour_color)
    TkcText.new(@c, [h_ratio*size, 0], :text=>@clock.div(4), 
                :tags=>[@tag, @mark_tag], :font=>@hour_font, 
                :anchor=>'w', :fill=>@hour_color)
    TkcText.new(@c, [0, h_ratio*size], :text=>@clock.div(2), 
                :tags=>[@tag, @mark_tag], :font=>@hour_font, 
                :anchor=>'n', :fill=>@hour_color)
    TkcText.new(@c, [-h_ratio*size, 0], :text=>@clock.div(4)*3, 
                :tags=>[@tag, @mark_tag], :font=>@hour_font, 
                :anchor=>'e', :fill=>@hour_color)

    [30.0, 60.0].each{|angle|
      rad = Math::PI * angle / 180.0
      x_base = size*Math::sin(rad)
      y_base = size*Math::cos(rad)

      x1 = 0.90*x_base
      y1 = 0.90*y_base

      x2 = 0.85*x_base
      y2 = 0.85*y_base

      TkcLine.new(@c, x1, y1, x2, y2, 
                  :tags=>[@tag, @mark_tag], 
                  :width=>@mark_width, :fill=>@mark_color)
      TkcLine.new(@c, x1, -y1, x2, -y2, 
                  :tags=>[@tag, @mark_tag], 
                  :width=>@mark_width, :fill=>@mark_color)
      TkcLine.new(@c, -x1, y1, -x2, y2, 
                  :tags=>[@tag, @mark_tag], 
                  :width=>@mark_width, :fill=>@mark_color)
      TkcLine.new(@c, -x1, -y1, -x2, -y2, 
                  :tags=>[@tag, @mark_tag], 
                  :width=>@mark_width, :fill=>@mark_color)

      #x3 = 0.92*x_base
      #y3 = 0.92*y_base
      x3 = h_ratio*x_base
      y3 = h_ratio*y_base

      if @clock == 24
        dh = angle.to_i/15
      else # @clock == 12
        dh = angle.to_i/30
      end

      TkcText.new(@c, x3, -y3, :text=>dh, 
                  :tags=>[@tag, @mark_tag], :font=>@hour_font, 
                  :anchor=>'sw', :fill=>@hour_color)
      TkcText.new(@c, x3, y3, :text=>@clock.div(2)-dh, 
                  :tags=>[@tag, @mark_tag], :font=>@hour_font, 
                  :anchor=>'nw', :fill=>@hour_color)
      TkcText.new(@c, -x3, y3, :text=>@clock.div(2)+dh, 
                  :tags=>[@tag, @mark_tag], :font=>@hour_font, 
                  :anchor=>'ne', :fill=>@hour_color)
      TkcText.new(@c, -x3, -y3, :text=>@clock-dh, 
                  :tags=>[@tag, @mark_tag], :font=>@hour_font, 
                  :anchor=>'se', :fill=>@hour_color)
    }

    if @clock == 24
      [15.0, 45.0, 75.0].each{|angle|
        rad = Math::PI * angle / 180.0
        x_base = size*Math::sin(rad)
        y_base = size*Math::cos(rad)

        x1 = 0.90*x_base
        y1 = 0.90*y_base

        x2 = 0.875*x_base
        y2 = 0.875*y_base

        TkcLine.new(@c, x1, y1, x2, y2, 
                    :tags=>[@tag, @mark_tag], 
                    :width=>@mark_width, :fill=>@submark_color)
        TkcLine.new(@c, x1, -y1, x2, -y2, 
                    :tags=>[@tag, @mark_tag], 
                    :width=>@mark_width, :fill=>@submark_color)
        TkcLine.new(@c, -x1, y1, -x2, y2, 
                    :tags=>[@tag, @mark_tag], 
                    :width=>@mark_width, :fill=>@submark_color)
        TkcLine.new(@c, -x1, -y1, -x2, -y2, 
                    :tags=>[@tag, @mark_tag], 
                    :width=>@mark_width, :fill=>@submark_color)
      }
    end

    #####################################

    TkcLine.new(@c, 0, -0.90*@size, 0, -0.85*@size, 
                :tags=>[@tag, @mark_tag], 
                :width=>@mark_width, :fill=>@mark_color)
    TkcLine.new(@c, 0.90*@size, 0, 0.85*@size, 0, 
                :tags=>[@tag, @mark_tag], 
                :width=>@mark_width, :fill=>@mark_color)
    TkcLine.new(@c, 0, 0.90*@size, 0, 0.85*@size, 
                :tags=>[@tag, @mark_tag], 
                :width=>@mark_width, :fill=>@mark_color)
    TkcLine.new(@c, -0.90*@size, 0, -0.85*@size, 0, 
                :tags=>[@tag, @mark_tag], 
                :width=>@mark_width, :fill=>@mark_color)

    TkcText.new(@c, [0, -0.92*@size], :text=>0, 
                :tags=>[@tag, @mark_tag], :font=>@mark_font, 
                :anchor=>'s', :fill=>@mark_color)
    TkcText.new(@c, [0.92*@size, 0], :text=>15, 
                :tags=>[@tag, @mark_tag], :font=>@mark_font, 
                :anchor=>'w', :fill=>@mark_color)
    TkcText.new(@c, [0, 0.92*@size], :text=>30, 
                :tags=>[@tag, @mark_tag], :font=>@mark_font, 
                :anchor=>'n', :fill=>@mark_color)
    TkcText.new(@c, [-0.92*@size, 0], :text=>45, 
                :tags=>[@tag, @mark_tag], :font=>@mark_font, 
                :anchor=>'e', :fill=>@mark_color)

    [30.0, 60.0].each{|angle|
      rad = Math::PI * angle / 180.0
      x_base = @size*Math::sin(rad)
      y_base = @size*Math::cos(rad)

      x1 = 0.90*x_base
      y1 = 0.90*y_base

      x2 = 0.85*x_base
      y2 = 0.85*y_base

      TkcLine.new(@c, x1, y1, x2, y2, 
                  :tags=>[@tag, @mark_tag], 
                  :width=>@mark_width, :fill=>@mark_color)
      TkcLine.new(@c, x1, -y1, x2, -y2, 
                  :tags=>[@tag, @mark_tag], 
                  :width=>@mark_width, :fill=>@mark_color)
      TkcLine.new(@c, -x1, y1, -x2, y2, 
                  :tags=>[@tag, @mark_tag], 
                  :width=>@mark_width, :fill=>@mark_color)
      TkcLine.new(@c, -x1, -y1, -x2, -y2, 
                  :tags=>[@tag, @mark_tag], 
                  :width=>@mark_width, :fill=>@mark_color)

      x3 = 0.92*x_base
      y3 = 0.92*y_base
      dh = angle.to_i/6

      TkcText.new(@c, x3, -y3, :text=>dh, 
                  :tags=>[@tag, @mark_tag], :font=>@mark_font, 
                  :anchor=>'sw', :fill=>@mark_color)
      TkcText.new(@c, x3, y3, :text=>30-dh, 
                  :tags=>[@tag, @mark_tag], :font=>@mark_font, 
                  :anchor=>'nw', :fill=>@mark_color)
      TkcText.new(@c, -x3, y3, :text=>30+dh, 
                  :tags=>[@tag, @mark_tag], :font=>@mark_font, 
                  :anchor=>'ne', :fill=>@mark_color)
      TkcText.new(@c, -x3, -y3, :text=>60-dh, 
                  :tags=>[@tag, @mark_tag], :font=>@mark_font, 
                  :anchor=>'se', :fill=>@mark_color)
    }


    [6, 12, 18, 24, 36, 42, 48, 54, 66, 72, 78, 84].each{|angle|
      rad = Math::PI * angle / 180.0
      x_base = @size*Math::sin(rad)
      y_base = @size*Math::cos(rad)

      x1 = 0.90*x_base
      y1 = 0.90*y_base

      x2 = 0.875*x_base
      y2 = 0.875*y_base

      TkcLine.new(@c, x1, y1, x2, y2, 
                  :tags=>[@tag, @mark_tag], :fill=>@submark_color)
      TkcLine.new(@c, x1, -y1, x2, -y2, 
                  :tags=>[@tag, @mark_tag], :fill=>@submark_color)
      TkcLine.new(@c, -x1, y1, -x2, y2, 
                  :tags=>[@tag, @mark_tag], :fill=>@submark_color)
      TkcLine.new(@c, -x1, -y1, -x2, -y2, 
                  :tags=>[@tag, @mark_tag], :fill=>@submark_color)
    }
  end

  def _create_hands
    hour_hand_len   = 0.70*@size
    minute_hand_len = 0.87*@size
    second_hand_len = 0.88*@size

    hour_hand_width   = 1.8*@cdot_size
    minute_hand_width = 1.0*@cdot_size
    second_hand_width = 1 # 0.4*@cdot_size

    @hour_hand_coords = [
      [0, -0.5*@cdot_size], 
      [hour_hand_width, -0.5*@cdot_size-hour_hand_width], 
      [hour_hand_width, -hour_hand_len+hour_hand_width], 
      [0, -hour_hand_len], 
      [-hour_hand_width, -hour_hand_len+hour_hand_width], 
      [-hour_hand_width, -0.5*@cdot_size-hour_hand_width], 
    ]
    @minute_hand_coords = [
      [0, -0.5*@cdot_size], 
      [minute_hand_width, -0.5*@cdot_size - minute_hand_width], 
      [minute_hand_width, -minute_hand_len+minute_hand_width], 
      [0, -minute_hand_len], 
      [-minute_hand_width, -minute_hand_len+minute_hand_width], 
      [-minute_hand_width, -0.5*@cdot_size-minute_hand_width], 
    ]
    @second_hand_coords = [
      [0, -0.5*@cdot_size], 
      [second_hand_width, -0.5*@cdot_size - second_hand_width], 
      [second_hand_width, -second_hand_len+second_hand_width], 
      [0, -second_hand_len], 
      [-second_hand_width, -second_hand_len+second_hand_width], 
      [-second_hand_width, -0.5*@cdot_size-second_hand_width], 
    ]

    @hour_hand = TkcPolygon.new(@c, @hour_hand_coords, 
                                :tags=>[@tag, @hand_tag], 
                                :outline=>@hour_hand_color, 
                                :fill=>@hour_hand_color)

    @minute_hand = TkcPolygon.new(@c, @minute_hand_coords, 
                                  :tags=>[@tag, @hand_tag], 
                                  :outline=>@minute_hand_color, 
                                  :fill=>@minute_hand_color)

    @second_hand = TkcPolygon.new(@c, @second_hand_coords, 
                                  :tags=>[@tag, @hand_tag], 
                                  :outline=>@second_hand_color, 
                                  :fill=>@second_hand_color)

    @center_dot = TkcOval.new(@c, 
                              [-@cdot_size, -@cdot_size], 
                              [@cdot_size, @cdot_size], 
                              :outline=>@cdot_color, :fill=>@cdot_color)
  end
  private :_create_hands

  def _raise_hands
    @hour_hand.raise
    @minute_hand.raise
    @second_hand.raise
    @center_dot.raise
  end
  private :_raise_hands

  def _raise_marks
    @mark_tag.raise
    @balloon_tag.raise
  end
  private :_raise_marks

  def set_hands(hh, mm, ss)
    ss_angle = Math::PI * ss / 30.0
    mm_angle = Math::PI * (mm + ss/60.0) / 30.0
    hh_angle = Math::PI * (hh + (mm + ss/60.0)/60.0) / (@clock.div(2))

    @second_hand.coords = @second_hand_coords.collect{|x, y|
      r = Math::hypot(y, x)
      a = Math::atan2(y, x) + ss_angle
      [Math::cos(a) * r, Math::sin(a) * r]
    }

    @minute_hand.coords = @minute_hand_coords.collect{|x, y|
      r = Math::hypot(y, x)
      a = Math::atan2(y, x) + mm_angle
      [Math::cos(a) * r, Math::sin(a) * r]
    }

    @hour_hand.coords = @hour_hand_coords.collect{|x, y|
      r = Math::hypot(y, x)
      a = Math::atan2(y, x) + hh_angle
      [Math::cos(a) * r, Math::sin(a) * r]
    }

    _raise_hands
    _raise_marks
  end

  def coords_to_time(x, y)
    return ((y < 0)? [0, 0]: [@clock.div(2), 0])  if x == 0
    if @clock == 24
      offset = (x<0&&y<0)? 1800.0: 360.0
      m_half = 720.0
    else # @clock == 12
      offset = (x<0&&y<0)? 900.0: 180.0
      m_half = 360.0
    end
    (offset + m_half*Math.atan2(y,x)/Math::PI).round.divmod(60)
  end

  def create_pie(hh, mm, span, color='red')
    if @clock == 24
      start  = 90.0 - (hh*60 + mm)/4.0  # 360.0*(hh*60+mm)/(24*60)
      extent = -span/4.0
    else # @clock == 12
      start  = 90.0 - (hh*60 + mm)/2.0  # 360.0*(hh*60+mm)/(12*60)
      extent = -span/2.0
    end

    pie = Clock::Pie.new(@c, @hour_circle_coords, :tags=>[@tag, @pie_tag], 
                         :outline=>'black', 'fill'=>color, 
                         :start=>start, :extent=>extent)

    pie.set_start_time(hh, mm)
    pie.set_end_time(*(hh*60 + mm + span).divmod(60))

    #pie.bind('Control-ButtonPress-3', proc{pie.destroy})
    pie.bind('ButtonPress-3', proc{pie.destroy})

    _raise_hands
    _raise_marks
    pie
  end

  def set_pie_extent(pie, span)
    if @clock == 24
      extent = -span/4.0
    else # @clock == 12
      extent = -span/2.0
    end
    hh, mm = pie.get_start_time
    pie.set_end_time(*(hh*60 + mm + span).divmod(60))
    pie[:extent] = extent
  end

  def set_pie_color(pie, col)
    pie[:fill] = col
  end
end

sched = Clock.new
sched.create_pie(0,0, 60, 'cyan')    #  60 minutes from 00:00
sched.create_pie(6,30, 280, 'green') # 280 minutes from 06:30
sched.create_pie(15,20, 90, 'blue')  #  90 minutes from 15:20

Tk.mainloop
