#
#  application_selector.rb
#
#    You have to select startup-applications 
#    and append them to `STARTUP_APPLICATIONS' array.
#
#    If STARTUP_APPLICATIONS is empty, Ruby/TkORCA will start 
#    with no daughter.
#
#    The following constants are availabe: 
#       STARTUP_APPLICATIONS
#       AUTH_STATUS
#       MOTHER
#       SYSTEM_DIR
#       SYSCONF_DIR
#       SCRIPT_DIR
#       ARGV
#       OPTION
#
#    You can use the constant `MOTHER' to start something like 
#    a `application launcher' on the mother wall.
#    The constant denotes the current mother object, 
#    but it will be removed after finishing evaluation of this file
#    ( STARTUP_APPLICATIONS and AUTH_STATUS will be removed too ).
#    If you need the mother object for the launcher, 
#    you have to keep the mother object on a local variable.
#
#    When you want to exit Ruby/TkORCA for some reasons, 
#    please call `TkORCA.exit'.
#
mother = MOTHER
wall = mother.wall

begin
  # load File.join(SYSTEM_DIR, 'wrap-test-demo-tbl.rb')
  load File.join(SCRIPT_DIR, 'wrap-test-demo-tbl.rb')
  demos = TkORCA_DEMOS

rescue Exception
  demos = [
    ['wrap-test-target.rb','Simple Demo: create a new toplevel',    false,4],
    ['pendulum.rb',        'Test of interactivity (Animation demo)',false,4], 
    ['pendulum2.rb',       'Yet another animation demo (multi-win)',false,4], 
    # ['figmemo_sample.rb',  'Put memo text on a image',              true, 4],
    ['vtk-test.rb',        'Test with VTK (Visualization Toolkit)', true, 4],
  ]
end

if ARGV.empty?
  sel_frame = TkFrame.new(wall, :relief=>:flat, :borderwidth=>3)

  TkLabel.new(sel_frame, 
              :text=>'Please select the demo which you want to try.', 
              :foreground=>'red').pack(:padx=>10, :pady=>5)

  selected_file = nil
  trusted_slave = false
  safe_level    = 4
  use_console   = false
  init_proc     = nil

  demos.each{|file, label, trust, safe, console, init|
    next unless file

    label   = file  unless label
    trust   = false unless trust
    safe    = 4     unless safe
    console = false unless console

    TkButton.new(sel_frame, :text=>label, #:anchor=>:w, 
                 :command=>proc{
                   selected_file = file
                   trusted_slave = trust
                   safe_level = safe
                   use_console = console
                   init_proc = init
                   sel_frame.destroy
                 }).pack(:fill=>:x)
  }

  TkFrame.new(sel_frame).pack(:pady=>5)

  TkButton.new(sel_frame, :text=>'QUIT', 
               :command=>proc{sel_frame.destroy}).pack(:fill=>:x)

  TkcWindow.new(wall, 40, 70, :anchor=>:nw, :window=>sel_frame)

  Tk.update
  sel_frame.thread_wait_destroy

  if selected_file
    selected_file = File.join(SCRIPT_DIR, selected_file)
    STARTUP_APPLICATIONS << [
      selected_file, trusted_slave, safe_level, use_console, init_proc
    ]
  else
    TkORCA.exit  # exit with finalize operation
  end

else # ARGV is not empty
  ARGV.each{|info|
    STARTUP_APPLICATIONS << 
      TkORCA::Parse_Application_Opt.parse_app_opt(info, demos)
  }
=begin
  ARGV.each{|info|
    file, trust, safe, console, init = info.split(',', 5)
    # In this case,  `init' means init-script filaname

    trust   = nil if trust   && trust.empty?
    safe    = nil if safe    && safe.empty?
    console = nil if console && console.empty?
    init    = nil if init    && init.empty?

    # get defaults from `demos' table (use infomation of first entry)
    info = demos.assoc(file)
    if info
      trust   = info[2] unless trust
      safe    = info[3] unless safe
      console = info[4] unless console
      init    = info[5] unless init
    end

    file = File.join(SCRIPT_DIR, file) unless file =~ %r{^[/~]}
    selected_file = File.expand_path(file)

    trust = TkComm.bool(trust) if trust
    trusted_slave = (trust.nil?)? false: trust

    safe_level = (safe)? safe.to_i: 4

    console = TkComm.bool(console) if console
    use_console = (console.nil?)? false: console

    init_script = init
    init_proc = (init_script)? proc{|ip| eval(init_script)}: nil

    STARTUP_APPLICATIONS << [
      selected_file, trusted_slave, safe_level, use_console, init_proc
    ]
  }
=end
=begin
  # file, trust, safe, init_cmd = ARGV
  file = ARGV.shift
  trust = ARGV.shift
  safe = ARGV.shift
  init_cmd = ARGV.shift
  console = ARGV.shift

  info = demos.assoc(file)
  if info
    trust   = info[2] 
    safe    = info[3]
    init    = info[4]
    console = info[5]
  end

  trust = TkComm.bool(trust) if trust
  trust = false if trust.nil?

  safe = (safe)? safe.to_i: 4

  init = (init_cmd)? proc{eval(init_cmd)}: nil

  console = TkComm.bool(console) if console
  console = false if console.nil?

  selected_file = File.expand_path(file)
  trusted_slave = trust
  safe_level = safe
  init_proc  = init
  use_console  = console

  STARTUP_APPLICATIONS << [
    selected_file, trusted_slave, safe_level, init_proc, use_console
  ]
=end
end

#=========================================================#

=begin
#wall.write_msg(500, 300, :text=><<EOT)
wall.write_msg(370, 170, :text=><<EOT)
The root window of the #{(trusted_slave)? 'slave': 'safeTk'} IP is 
embedded in Master IP's frame widget.

You can move the root window by 
'Button-1 + Motion' on the titlebar.
Master IP works like as a window manager.
(When 'Alt + Button-1 + Motion', stack order is not changed).


No window manager on the VNC server.
Running one Ruby/Tk process only.
EOT
#'
=end

#=========================================================#

# STARTUP_APPLICATIONS << [File.join(SCRIPT_DIR, 'pendulum2.rb'), false, 4, nil, false]

# mother.new_daughter(true, 4).load_script(File.join(SCRIPT_DIR, 'wrap-test-target.rb'))
# d2 = mother.new_daughter(false, 4)
# d2.__eval_proc__{TkORCA::Console.create('Console').lower}
# d2.load_script(File.join(SCRIPT_DIR, 'pendulum2.rb'))
