#!/usr/bin/env ruby

class_name = ARGV.shift

fields = []
while s = gets
  s.chomp!
  s.strip!
  fields << s unless s.empty?
end

######################
print <<SCRIPT
require 'tk'

class #{class_name} < TkWindow
  include TkComposite

  def initialize_composite(keys = {})
    @fields_f = TkFrame.new(@frame)
    @buttons_f = TkFrame.new(@frame)
    @fields = {}

    # convert hash keys
    keys = _symbolkey2str(keys)

SCRIPT

######################
fields.each{|name|
  print <<FIELDS
    Tk.grid(TkLabel.new(@fields_f, :text=>'#{name}', :anchor=>:e), 
            @fields['#{name}'] = TkEntry.new(@fields_f, 
                                             :textvariable=>TkVariable.new), 
            :sticky=>'news')
FIELDS
}
print "\n"

######################
print <<'SCRIPT'
    @fields_f.grid_columnconfigure(0,  :weight=>0)
    @fields_f.grid_columnconfigure(1,  :weight=>1)

    # buttons
    TkFrame.new(@buttons_f, :height=>2, :relief=>:sunken, :borderwidth=>1, 
                :background=>'gray50').pack(:fill=>:x, :expand=>false, 
                                           :padx=>5, :pady=>2)
    @b_ok     = TkButton.new(@buttons_f, 
                             :command=>proc{@ok_cmd.call if @ok_cmd}, 
                             :text=>'OK').pack(:expand=>true, :side=>:left)
    @b_clear  = TkButton.new(@buttons_f, :command=>proc{clear}, 
                             :text=>'clear').pack(:expand=>true, :side=>:left)

    # layout
    @fields_f.pack(:fill=>:both, :expand=>false, :side=>:top)
    buttonframe(keys.delete('buttonframe'){true})
  end

  def [](key)
    key = key.to_s
    fail IndexError,"unknown field `#{key}'"  unless @fields.key?(key)
    @fields[key].value
  end
  alias value []

  def []=(key, val)
    key = key.to_s
    fail IndexError,"unknown field `#{key}'"  unless @fields.key?(key)
    @fields[key].value = val
  end

  def values
    vals = {}
    @fields.each{|key, entry| vals[key] = entry.value}
    vals
  end

  def values=(hash)
    hash.each{|key,val|
      fail IndexError,"unknown field `#{key}'"  unless @fields.key?(key)
      @fields[key].value = val
    }
    hash
  end

  def clear
    @fields.each_value{|entry| entry.delete(0, :end)}
    self
  end

  def command(cmd = Proc.new)
    @ok_cmd = cmd
  end

  def add_button(name, cmd = Proc.new)
    TkButton.new(@buttons_f, :command=>cmd, 
                 :text=>name).pack(:expand=>true, :side=>:left)
  end

  def buttonframe(mode)
    if mode
      @buttons_f.pack(:fill=>:x, :expand=>false, :side=>:bottom)
    else
      @buttons_f.unpack
    end
  end
end
SCRIPT

print <<SCRIPT
if $0 == __FILE__
  dialog = #{class_name}.new.pack(:fill=>:both, :expand=>true)
  dialog.command{p dialog.values; exit}
  Tk.mainloop
end
SCRIPT
