# Aquativo - Copyright (C) 2004 Pat Thoyts <patthoyts@users.sourceforge.net>
#
# Import the Aquativo Gtk theme (C) Andrew Wyatt, FEWT Software
# Original: http://www.fewt.com
# Link: http://art.gnome.org/themes/gtk2/432.php
#
# You will need to fetch the theme package and extract it under the 
# demos/themes directory and maybe modify the demos/themes/pkgIndex.tcl
# file.
#
# $Id: Aquativo.tcl,v 1.2 2004/10/30 00:06:14 patthoyts Exp $

package require Tk 8.4;                 # minimum version for Tile
package require tile;                   # depends upon tile
#package require tile::image
package require Img

namespace eval tile {
    namespace eval theme {
        namespace eval Aquativo {
            variable version 0.0.3
        }
    }
}

namespace eval tile::theme::Aquativo {

    set imgdir [file join [file dirname [info script]] Aquativo-1.5 gtk-2.0]
    array set I [tile::LoadImages $imgdir {
        horizontal_trough.png vertical_trough.png
        button_normal_in.png button_prelight_out.png
        button_insensitive_out.png button_normal_out.png
        check_in.png check_out.png
        option_in.png option_out.png
        menubar_option.png menubar_option_arrow.png 
        menubar_option_arrow_insensitive.png
        scrollbar_prelight_horizontal.png scrollbar_horizontal.png
        scrollbar_prelight_vertical.png scrollbar_vertical.png
        arrow_up_clicked.png arrow_up_insensitive.png arrow_up_normal.png
        arrow_down_clicked.png arrow_down_insensitive.png arrow_down_normal.png
        arrow_left_clicked.png arrow_left_insensitive.png arrow_left_normal.png
        arrow_right_clicked.png arrow_right_insensitive.png 
        arrow_right_normal.png
        notebook_bottom.png notebook_active_bottom.png
        
    }]

    style theme create Aquativo -parent alt -settings {

        # -----------------------------------------------------------------
        # Defaults
        # -----------------------------------------------------------------

        style default "." -font TkDefaultFont -background #fafafa 
        style default TButton -padding {1 1} -width -11

        # -----------------------------------------------------------------
        # Troughs
        # -----------------------------------------------------------------

        style element create trough image $I(horizontal_trough) \
            -border 3
        style element create Vertical.Scrollbar.trough \
            image $I(vertical_trough) -border 2
        style element create Vertical.Scale.trough \
            image $I(vertical_trough) -border 2
        style element create Vertical.Progress.trough \
            image $I(vertical_trough) -border 2

        # -----------------------------------------------------------------
        # Buttons, Checkbuttons and Radiobuttons
        # -----------------------------------------------------------------

        style layout TButton {
            Button.background
            Button.button -children {
                Button.focus -children {
                    Button.label
                }
            }
        }

        style element create button image $I(button_normal_out) \
            -border {4 13 4 13} -sticky news \
            -map [list {pressed !disabled} $I(button_normal_in) \
                      {active !disabled}  $I(button_prelight_out) \
                      disabled            $I(button_insensitive_out)]
        
        style element create Checkbutton.indicator image $I(check_out) \
            -border 2 -width 20 -sticky w \
            -map [list {selected !disabled} $I(check_in)]

        style element create Radiobutton.indicator image $I(option_out) \
            -border 2 -width 20 -sticky w \
            -map [list {selected !disabled} $I(option_in)]

        # -----------------------------------------------------------------
        # Menubuttons:
        # -----------------------------------------------------------------

        style element create Menubutton.button image $I(menubar_option) \
            -border {7 10 29 15} -padding {7 4 29 4} -sticky news
        
        style element create Menubutton.indicator \
            image $I(menubar_option_arrow) \
            -width 11 -sticky w -padding {0 0 18 0} \
            -map [list disabled $I(menubar_option_arrow_insensitive)]
        
        # -----------------------------------------------------------------
        # Scrollbar
        # -----------------------------------------------------------------

        style default TScrollbar -width 16

        style element create Horizontal.Scrollbar.thumb \
            image $I(scrollbar_horizontal) \
            -border 3 -width 16 -height 16 -sticky news \
            -map [list {active !disabled} $I(scrollbar_prelight_horizontal)]
        
        style element create Vertical.Scrollbar.thumb \
            image $I(scrollbar_vertical)\
            -border 3 -width 16 -height 15 -sticky news \
            -map [list {active !disabled}  $I(scrollbar_prelight_vertical)]

        # -----------------------------------------------------------------
        # Scale
        # -----------------------------------------------------------------

        style element create Horizontal.Scale.slider \
            image $I(scrollbar_horizontal) \
            -border 3 -width 30 -height 16 \
            -map [list {active !disabled} $I(scrollbar_prelight_horizontal)]

        style element create Vertical.Scale.slider \
            image $I(scrollbar_vertical) \
            -border 3 -width 16 -height 30 \
            -map [list {active !disabled} $I(scrollbar_prelight_vertical)]

        # -----------------------------------------------------------------
        # Progress
        # -----------------------------------------------------------------

        # should use ::tile::Aquativo::_i_progressbar but this is too big.
        style element create bar \
            image $I(scrollbar_prelight_horizontal) \
            -border 3
        style element create Vertical.Progress.bar \
            image $I(scrollbar_prelight_vertical) \
            -border 3

        # ------------------------------------------------------------------
        # Arrows
        # ------------------------------------------------------------------

            style element create uparrow image $I(arrow_up_normal) \
                -map [list {pressed !disabled} $I(arrow_up_clicked) \
                          disabled             $I(arrow_up_insensitive)]
            style element create downarrow image $I(arrow_down_normal) \
                -map [list {pressed !disabled} $I(arrow_down_clicked) \
                          disabled             $I(arrow_down_insensitive)]
            style element create leftarrow image $I(arrow_left_normal) \
                -map [list {pressed !disabled} $I(arrow_left_clicked) \
                          disabled             $I(arrow_left_insensitive)]
            style element create rightarrow image $I(arrow_right_normal) \
                -map [list {pressed !disabled} $I(arrow_right_clicked) \
                          disabled             $I(arrow_right_insensitive)]

        # -----------------------------------------------------------------
        # Notebook parts
        # -----------------------------------------------------------------
    
        style element create tab image $I(notebook_active_bottom) \
            -sticky news \
            -map [list {selected !disabled} $I(notebook_bottom)]

        # -----------------------------------------------------------------
        # Frames parts
        # -----------------------------------------------------------------
        
        # This make it REALLY SLOW
        #style element create Frame.background image $I(window_background)
        #style element create Labelframe.background image $I(window_backround)

    }    
}

# -------------------------------------------------------------------------

package provide tile::theme::Aquativo $::tile::theme::Aquativo::version

# -------------------------------------------------------------------------
