#
#  application_controller.rb : 
#
#                             by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
require 'thread'
require 'socket'

class Application_Controller
  class RECONNECT_COMMANDER < IOError; end

  def to_commander(str)
    if (io = @to_commander)
      @to_commander_thread = Thread.new{
        begin
          io.write(str)
          io.flush
        rescue Exception
          # cannot continue I/O operation
          @to_commander = nil
        end
      }
      @to_commander_thread.value
    else
      @to_commander_thread = nil
      nil
    end
  end

  def from_commander
    if (io = @from_commander)
      @from_commander_thread = Thread.new{
        begin
          io.readline  # trust controller user ( == system master, probably )
        rescue RECONNECT_COMMANDER
          nil
        rescue Exception
          # cannot continue I/O operation
          @from_commander = nil
          nil
        end
      }
    else
      @from_commander_thread = Thread.new{
        begin
          sleep
        rescue RECONNECT_COMMANDER
          nil
        end
      }
    end
    @from_commander_thread.value
  end
  private :to_commander, :from_commander

  ######################################

  def initialize(tkorca_pid, to_irb, from_irb, log_recv)
    @tkorca_pid  = tkorca_pid

    @to_commander   = nil
    @from_commander = nil

    @to_commander_thread   = nil
    @from_commander_thread = nil

    @to_irb   = to_irb
    @from_irb = from_irb
    @log_recv = log_recv

    @sender = Thread.new{
      loop {
        str = from_commander()

        begin
           if str
             @to_irb.write(str.to_s.chomp << "\n")
             @to_irb.flush
           end
        rescue IOError, Errno::EXXX
          # cannot continue I/O operation
          break
        rescue Exception
          # ignore error
        end
      }
    }

    @receiver = Thread.new{
      loop {
        begin
          str = @from_irb.readpartial(1024)
        rescue IOError, Errno::EXXX
          # cannot continue I/O operation
          break
        rescue Exception
          # ignore error
        end

        to_commander(str.to_s) if str
      }
    }

    @logger = Thread.new{
      buffer = ''
      loop {
        begin
          # Each log message must have "\n" at last.
          @log_recv.readpartial(1024).split(/^/).each{|line|
            buffer << line
            if buffer.chomp! || buffer.length > 2000 
              LOGGER.raw_write(buffer)
              buffer.replace('')
            end
          }
        rescue IOError, Errno::EXXX
          # cannot continue I/O operation
          break
        rescue Exception
          # ignore error
        end
      }
    }
  end

  def connect(get_cmds, put_msg)
    unless get_cmds.respond_to?(:readline)
      LOGGER.log("TypeError: first argument must respond to `readline'")
      fail TypeError, "first argument must respond to `readline'"
    end
    unless put_msg.respond_to?(:write)
      LOGGER.log("TypeError: second argument must respond to `write'")
      fail TypeError, "second argument must respond to `write'"
    end

    @to_commander   = put_msg
    @from_commander = get_cmds

    if @to_commander_thread
      begin
        @to_commander_thread.raise(RECONNECT_COMMANDER)
      rescue Exception
      end
    end

    if @from_commander_thread
      begin
        @from_commander_thread.raise(RECONNECT_COMMANDER)
      rescue Exception
      end
    end

    begin
      Process.kill(SERVER_PARAM['hangup_signal'], @tkorca_pid)
    rescue Exception
    end

    self
  end

  def disconnect
    (@to_commander.close rescue nil)   if @to_commander
    (@from_commander.close rescue nil) if @from_commander

    @to_commander   = nil
    @from_commander = nil

    begin
      Process.kill(SERVER_PARAM['hangup_signal'], @tkorca_pid)
    rescue Exception
    end

    if @to_commander_thread
      begin
        @to_commander_thread.raise(RECONNECT_COMMANDER)
      rescue Exception
      end
    end

    if @from_commander_thread
      begin
        @from_commander_thread.raise(RECONNECT_COMMANDER)
      rescue Exception
      end
    end

    self
  end

  def controller_close
    @to_irb.flush rescue nil

    if @to_irb.kind_of?(BasicSocket)
      @to_irb.shutdown(Socket::SHUT_WR) rescue nil
    end

    @to_irb.close rescue nil

    if @from_irb.kind_of?(BasicSocket)
      @from_irb.shutdown(Socket::SHUT_RD) rescue nil
    end

    @from_irb.close rescue nil

    @sender.kill   rescue nil
    @receiver.kill rescue nil

    self
  end
  def controller_closed?
    @to_irb.closed? || ! @sender.alive?
  end

  def logger_close
    if @log_recv.kind_of?(BasicSocket)
      @log_recv.shutdown(Socket::SHUT_RD) rescue nil
    end

    @log_recv.close rescue nil

    @logger.kill  rescue nil
    self
  end
  def logger_closed?
    @log_recv.closed? || ! @logger.alive?
  end

  def kill
    controller_close
    logger_close
    self
  end
end
