#
#  application_process.rb : create a process for a tkorca family
#
#                             by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
require 'socket'
require 'server/application_options.rb'

module Application_Process_Handler
  def exec_application_process(display_num, xauth_file, temp_home, 
                               ios, *mod_args)
    ruby_bin = SERVER_PARAM['ruby_bin']
    tkorca = SERVER_PARAM['tkorca_path']

    options = TkORCA_SERVICE_DEFAULT_OPTIONS.dup
    options.concat(TkORCA_SERVICE_OPTIONS)
    until mod_args.empty?
      arg = mod_args.shift
      break if arg == '--'
      options << arg
    end

    applications = TkORCA_SERVICE_DEFAULT_APPLICATIONS.dup
    applications.concat(TkORCA_SERVICE_APPLICATIONS)
    applications.concat(mod_args)
    applications.unshift('--') unless applications.empty?

    if SERVER_PARAM['app_root_dir']
      begin
        Dir.chroot(SERVER_PARAM['app_root_dir'])
      rescue Exception => e
        LOGGER.log("ID[#{@connection_id}]" + 
                     " cannot chroot `#{SERVER_PARAM['app_root_dir']}';" + 
                     " Error: #{e.class}: #{e.message}")
        raise e
      end
    end

    if USE_ID_CHANGE_PRIVILEGE
      begin
        Process::GID.change_privilege(SERVER_PARAM['group'])
        Process::UID.change_privilege(SERVER_PARAM['user'])
      rescue Exception => e
        LOGGER.log("ID[#{@connection_id}] cannot change privilege to " + 
                     "#{SERVER_PARAM['user']}/#{SERVER_PARAM['group']};" + 
                     " Error: #{e.class}: #{e.message}")
        raise e
      end
    end

    logger_r,   logger_w   = ios[0]  # LOGGER
    to_irb_r,   to_irb_w   = ios[1]  # TO IRB
    from_irb_r, from_irb_w = ios[2]  # FROM IRB

    from_irb_w.sync = true

    $stdin.reopen(to_irb_r)
    $stdout.reopen(from_irb_w)
    $stderr.reopen(from_irb_w)

    options << '--home_dir' << temp_home if temp_home
    options << '--interactive'  if SERVER_PARAM['interactive']
    options << '--hangup_signal' << SERVER_PARAM['hangup_signal'].to_s
    options << '--log_prefix' << "ID[#{@connection_id}]:"
    options << '--log_fileno' << logger_w.fileno.to_s

    cmd_line = [ruby_bin, tkorca].concat(options).concat(applications)

    ENV['HOME'] = temp_home if temp_home
    ENV['XAUTHORITY'] = xauth_file
    ENV['DISPLAY'] = ":#{display_num}"
    #ENV['DISPLAY'] = ":#{display_num}.0"

    LOGGER.log("ID[#{@connection_id}] exec", cmd_line.join(' ').inspect)

    # close unnecessary I/O objects
    ObjectSpace.each_object{|obj|
      if obj.kind_of?(IO)
        unless obj == logger_w || 
                  obj == $stdin || obj == $stdout || obj == $stderr
          obj.close rescue nil
        end
      end
    }

    exec(*cmd_line)
  end

  def fork_tkorca(display_num, xauth_file, temp_home, ios, *mod_args)
    fork{ exec_application_process(display_num, xauth_file, temp_home, 
                                   ios, *mod_args) }
#    fork{ exec_application_process(display_num, xauth_file, temp_home, 
#                                   ios, *mod_args) }
  end
end
