#
#  controller_command.rb : 
#
#                             by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
module TkORCA_Controller; end
class << TkORCA_Controller
  # command argments is 
  #   <original commanline string (stripped)>, <cmd string>, <arguments...>
  def op_list(cmd_str, cmd, *id_list)
    if id_list.empty?
      id_list = TkORCA_Service::Connection_ID_table.keys.sort
    else
      id_list = id_list.map{|id_str| Integer(id_str)}.sort
    end

    put_msg("--- list of current connections ---\n")
    id_list.each{|id|
      if (app = TkORCA_Service::Connection_ID_table[id])
        put_msg("ID[#{id}]: connection #{app.peer_info}" + 
                  " on display :#{app.display_num}" + 
                  " from #{app.startup_time.strftime('%Y-%m-%d %H:%M:%S')}\n")
      end
    }
    put_msg("-----------------------------------\n")

    if @application && (io = @to_app)
      io.write(%Q(""\n))
      io.flush
      false
    else
      true
    end
  end

  def op_logging(cmd_str, cmd, *tokens)
    msg = cmd_str.split(' ', 2) # to keep speces of original
    LOGGER.log("[user:#{@controller_username}] #{msg}")
    true
  end

  def op_kill(cmd_str, cmd, id_str = nil)
    unless id_str
      if (app = @application)
        app.kill
      else
        put_msg("Error: still not connect to service\n")
      end
      return true
    end

    id = Integer(id_str)
    if (app = TkORCA_Service::Connection_ID_table[id])
      app.kill
    else
      put_msg("Error: no running service which has ID == #{id}\n")
    end
    true
  end

  def op_connect(cmd_str, cmd, id_str)
    if (@application)
      disconnect_application()
    end

    id = Integer(id_str)
    connect_application(id)
    false
  end

  def op_disconnect(cmd_str, cmd)
    if @application
      disconnect_application()
    else
      disconnect_cmd_io_sock()
    end
    true
  end

  def op_exit(cmd_str, cmd)
    if @application
      disconnect_application()
    end
    disconnect_cmd_io_sock()
    true
  end

  def op_kill_server(cmd_str, cmd)
    LOGGER.log("User `#{@controller_username}': shutdown server")
    put_msg("shutdown the server\n")
    op_exit(cmd_str, cmd)
    Process.kill(:TERM, Process.pid)
  end
end

###################################

module TkORCA_Controller
  CONTROL_COMMAND_TABLE = {
    'list'        => [self.method(:op_list), 'show working services'], 
    'log'         => [self.method(:op_logging), 'write message to logfile'], 
    'kill'        => [self.method(:op_kill), 
                      'kill [id] : kill the connected service or id'], 
    'connect'     => [self.method(:op_connect), 
                      'connect <id> : connect to service <id>'], 
    'disconnect'  => [self.method(:op_disconnect), 
                      'disconnect from connected service'], 
    'exit'        => [self.method(:op_exit), 'disconnect from server'], 
    'quit'        => [self.method(:op_exit), 'disconnect from server'], 
    'kill_server' => [self.method(:op_kill_server), 'shutdown server'], 
    'shutdown'    => [self.method(:op_kill_server), 'shutdown server'], 
  }
end
