#
#  controller_passwd.rb : 
#
#                             by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
#
#    controller_passwd.rb <passwd-file> <sub-cmd> [ <arg> ... ]
#
#        <sub-cmd> ::  admin <user>
#                      auth  <user> <passwd> [<admin>]
#                      add   <user> <passwd> [<admin>]
#                      mod   <user> <old-passwd> <new-passwd>
#                      del   <user>
#
module TkORCA_Controller_Auth
  # MD5 supprt?
  begin
    if 'abcdefgh'.crypt('$1$12345678$').length > 13
      USE_MD5_CRYPT = true
    else
      USE_MD5_CRYPT = false
    end
  rescue
    USE_MD5_CRYPT = false
  end
end

class << TkORCA_Controller_Auth
  def read(passwd_file)
    @file = passwd_file
    @table = {}
    @admin = {}

    unless FileTest.exist?(@file)
      print "password file `#{@file}' does not exist. "
      print "that will be created.\n"
      open(@file,'w',0600){}
    end

    IO.foreach(@file){|line|
      user, passwd, admin = line.strip.chomp.split(':')
      @table[user] = passwd if passwd
      @admin[user] = 'admin' if admin && admin =~ /^[aA]/
    }
  end

  def admin?(user)
    (@admin[user])? true: false
  end

  def auth(user, passwd, admin=false)
    raise RuntimeError, "password file is not selected" unless @table
    unless @table[user]
      false
    else
      ((!admin || @admin[user]) && 
         (@table[user] == passwd.crypt(@table[user])))? true: false
    end
  end

  def _new_passwd(user, passwd)
    if self::USE_MD5_CRYPT
      salt = 
        '$1$' + 
        (1..8).collect{rand(64)}.pack("C*").tr("\x00-\x3f", "A-Za-z0-9./") + 
        '$'
    else
      salt = (1..2).collect{rand(64)}.pack("C*").tr("\x00-\x3f", "A-Za-z0-9./")
    end

    @table[user] = passwd.crypt(salt)
  end
  private :_new_passwd

  def add(user, passwd, admin=false)
    raise RuntimeError, "password file is not selected" unless @table
    raise RuntimeError, "user `#{user}' is already registered" if @table[user]

    _new_passwd(user, passwd)

    @admin[user] = 'admin' if admin

    open(@file, 'a'){|fd|
      fd.puts("#{user}:#{@table[user]}:#{@admin[user]||'user'}")
    }
    self
  end

  def _write_table
    open(@file, 'w'){|fd|
      @table.keys.sort.each{|u|
        fd.puts("#{u}:#{@table[u]}:#{@admin[u]||'user'}")
      }
    }
  end
  private :_write_table

  def mod(user, old_passwd, new_passwd, admin=false)
    return add(user, new_passwd) unless @table.has_key(user)

    unless auth(user, old_passwd)
      raise RuntimeError, "no permission to change passwd for user `#{user}'"
    end

    _new_passwd(user, passwd)

    if admin
      @admin[user] = 'admin'
    else
      @admin.delete(user)
    end

    _write_table()
  end

  def del(user)
    raise RuntimeError, "unknown user `#{user}'" unless @table.delete(user)
    _write_table()
    self
  end
end

if __FILE__ == $0
  passwd_file = ARGV.shift
  cmd = ARGV.shift
  user = ARGV.shift
  raise ArgumentError, "too few command line arguments" unless user

  TkORCA_Controller_Auth.read(passwd_file)

  case cmd
  when 'admin'
    p TkORCA_Controller_Auth.admin?(user)

  when 'auth'
    passwd = ARGV.shift
    admin = ARGV.shift
    raise ArgumentError, "too few arguments for `auth'" unless passwd
    p TkORCA_Controller_Auth.auth(user, passwd, admin)

  when 'add'
    passwd = ARGV.shift
    admin = ARGV.shift
    raise ArgumentError, "too few arguments for `add'" unless passwd
    TkORCA_Controller_Auth.add(user, passwd, admin)
    print "add user `#{user}' to file `#{passwd_file}'"

  when 'mod'
    old_passwd = ARGV.shift
    new_passwd = ARGV.shift
    admin = ARGV.shift
    raise ArgumentError, "too few arguments for `mod'" unless new_passwd
    TkORCA_Controller_Auth.mod(user, old_passwd, new_passwd, admin)
    print "password of user `#{user}' was changed"

  when 'del'
    TkORCA_Controller_Auth.del(user)
    print "delete user `#{user}' from file `#{passwd_file}'"

  else
    raise ArgumentError, "unknown command `#{cmd}'"
  end
end
