#
#  create_temp_home.rb : create temporal home directory
#
#                             by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
require 'server/temp_home_options.rb'

module Temporal_Home_Handler
  def get_temp_home_parent
    if SERVER_PARAM['checked_temp_home_parent']
      parent_dir = SERVER_PARAM['checked_temp_home_parent']
      return parent_dir if FileTest.directory?(parent_dir)
    end

    if SERVER_PARAM['temp_home_parent']
      parent_dir = SERVER_PARAM['temp_home_parent']
    elsif Object.const_defined?(:DEFAULT_TEMP_HOME_PARENT)
      parent_dir = File.expand_path(DEFAULT_TEMP_HOME_PARENT)
    elsif SERVER_PARAM['tmp_dir']
      parent_dir = File.join(SERVER_PARAM['tmp_dir'], 'tkorca_temporal_homes')
    else
      parent_dir = File.join(File.expand_path(Dir.tmpdir), 
                             'tkorca_temporal_homes')
    end

    unless FileTest.exist?(parent_dir)
      Dir.mkdir(parent_dir)
    end

    unless FileTest.directory?(parent_dir)
      raise Errno::ENOTDIR, "Not a directory - #{parent_dir}"
    end

    begin
      # if available
      mode = File.stat(parent_dir).mode & 0777
      if mode != 0311
        File.chmod(0311, parent_dir) # <==  d-wx--x--x
      end
    rescue
      # ignore error
    end

    SERVER_PARAM['checked_temp_home_parent'] = parent_dir  # for later use

    parent_dir
  end

  def get_temp_home_dirname
    parent_dir = get_temp_home_parent()

    #####################################

    path = nil
    begin
      path = File.join(parent_dir, 
                       #                                    0000000001111111
                       #                                    1234567890123456
                       'home%d_%016x' % [Process.pid,rand(0xffffffffffffffff)]
                       )
    end while(FileTest.exist?(path))
    path
  end

  def create_temporal_home(temp_home_permission = 0700)
    # get a new temporal home dirname
    temp_home = get_temp_home_dirname()

    # create directory
    Dir.mkdir(temp_home, temp_home_permission)

    temp_home
  end

  def _remove_dir(dir)
    File.chmod(0777, dir)
    Dir.foreach(dir){|name|
      next if name == '.' || name == '..'
      path = File.join(dir, name)
      if FileTest.directory?(path)
        _remove_dir(path)
      else
        File.delete(path)
      end
    }
    Dir.delete(dir)
    dir
  end

  def remove_temporal_home(home)
    _remove_dir(home) if home
  end
end
