#
#  create_xauth.rb : create Xauthority file
#
#                             by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
require 'server/xauth_options.rb'

module Xauth_File_Handler
  def get_xauth_dir
    if SERVER_PARAM['checked_xauth_dir']
      xauth_dir = SERVER_PARAM['checked_xauth_dir']
      return xauth_dir if FileTest.directory?(xauth_dir)
    end

    if SERVER_PARAM['xauth_dir']
      xauth_dir = SERVER_PARAM['xauth_dir']
    elsif Object.const_defined?(:DEFAULT_XAUTH_DIR)
      xauth_dir = File.expand_path(DEFAULT_XAUTH_DIR)
    elsif SERVER_PARAM['tmp_dir']
      xauth_dir = File.join(SERVER_PARAM['tmp_dir'], 'tkorca_xauth_files')
    else
      xauth_dir = File.join(File.expand_path(Dir.tmpdir), 'tkorca_xauth_files')
    end

    unless FileTest.exist?(xauth_dir)
      Dir.mkdir(xauth_dir)
    end

    unless FileTest.directory?(xauth_dir)
      raise Errno::ENOTDIR, "Not a directory - #{xauth_dir}"
    end

    begin
      # if available
      mode = File.stat(xauth_dir).mode & 0777
      if mode != 0311
        File.chmod(0311, xauth_dir) # <==  d-wx--x--x
      end
    rescue
      # ignore error
    end

    SERVER_PARAM['checked_xauth_dir'] = xauth_dir  # for later use

    xauth_dir
  end

  def get_xauth_filename
    xauth_dir = get_xauth_dir()

    #####################################

    path = nil
    begin
      path = File.join(xauth_dir, 
                       #                                     0000000001111111
                       #                                     1234567890123456
                       'xauth%d_%016x' % [Process.pid,rand(0xffffffffffffffff)]
                       )
    end while(FileTest.exist?(path))
    path
  end

  def create_xauth_cookie
    #                00000000011111111112222222222333
    #                12345678901234567890123456789012
    '%032x' % rand(0xffffffffffffffffffffffffffffffff)
  end

  def create_xauth_file(display_num, xauth_file_permission = 0400)
    if SERVER_PARAM['xauth_path']
      xauth_cmd = SERVER_PARAM['xauth_path']
    else
      xauth_cmd = File.expand_path(DEFAULT_XAUTH_PATH)
    end

    # get a new xauth filename
    xauth_file = get_xauth_filename()

    #ENV['XAUTHORITY'] = xauth_file

    # create the empty xauth file to inhibit `creating' message from xauth 
    open(xauth_file, 'w'){}

    # for localhost :: this will create "<host>/unix:<display>" entry.
    system("#{xauth_cmd} -qf #{xauth_file} " + 
             "add :#{display_num} MIT-MAGIC-COOKIE-1 #{create_xauth_cookie()}")

    # for remote hosts :: usually, this is not necessary.
    system("#{xauth_cmd} -qf #{xauth_file} " + 
             "add #{SERVER_PARAM['hostname']}:#{display_num} " + 
             "MIT-MAGIC-COOKIE-1 #{create_xauth_cookie()}")

    File.chmod(xauth_file_permission, xauth_file)

    xauth_file
  end
end

