#
#  logger.rb : 
#
#                             by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
class TkORCA_LOGGER
  def initialize(logfile, verbose=true)
    @verbose = (verbose)? true: false
    @logfilter = nil
    @logfile = logfile
    @out = open(@logfile, 'a')
    unless @out.flock(File::LOCK_EX|File::LOCK_NB)
      raise RuntimeError, "fail to lock logfile `#{@logfile}'." + 
        ' Another Ruby/TkORCA server process may be running.'
      exit!(1)
    end
    self.log('logfile open') if @verbose
  end

  def restart(logfile = nil)
    self.close
    @logfile = logfile if logfile
    @out.reopen(@logfile, 'a')
    unless @out.flock(File::LOCK_EX|File::LOCK_NB)
      raise RuntimeError, "fail to lock logfile `#{@logfile}'." + 
        ' Another Ruby/TkORCA server process may be running.'
      exit!(1)
    end
    self.log('logfile reopen')
    self
  end

  def close
    self.log('logfile close') if @verbose
    @out.close
  end

  def closed?
    @out.closed?
  end

  attr_accessor :verbose
  attr_accessor :logfilter  # select log, or invoke commands

  def log(*strings)
    str = strings.join(' ').chomp
    if !@logfilter || @logrilter.call(str)
      @out.write(Time.now.strftime('%Y-%m-%d %H:%M:%S> ') + str + "\n")
      @out.flush
    end
    self
  end

  def raw_write(*strings)
    str = strings.join(' ').chomp
    if !@logfilter || @logrilter.call(str)
      @out.write(str + "\n")
      @out.flush
    end
    self
  end
end
