#
#  server/server_opt_parse.rb : command line option parser
#
#    You must make a hash `SERVER_OPTION' which keeps option keys and values.
#
#                           by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
OPTION = {
  # flags
  'version'        => false,  # -v
  'debug_mode'     => false,  # -d  
  'reverse_lookup' => false,  # -L
  'interactive'    => true,   # -I
  'use_tmp_home'   => true,   # -H

  # numeric
  'port'           => -1,     # -p
  'listen'         => -1,     # -l

  'control_port'   => -1,     # -c

  # string
  'hostaddr'       => "",     # -h

  'user'           => nil,    # -u
  'group'          => nil,    # -g
  'server_user'    => nil,    # -U
  'server_group'   => nil,    # -G

  'serverrc'       => nil,    # -s

  'app_root_dir'   => nil,    # -r
  'tmp_dir'        => nil,    # -t
  'tmp_home_dir'   => nil,    # -T

  'logfile'        => nil,    # -f

  'null_device'    => nil,    # -n

  'hangup_signal'  => nil,    # -S

  'ruby_bin'       => nil,    # -b

  'control_passwd' => nil,    # -C  (file path)

  'xauth_path'     => nil,    # -a
  'xauth_dir'      => nil,    # -A
  'xvnc_path'      => nil,    # -x
  'tkorca_path'    => nil,    # -o

  # Xvnc options (override DEFAULT_XVNC_OPTION_LIST on `xvnc_options.rb')
  'rfbauth_passwd' => nil,    # -R  (file path)
  'desktop_name'   => nil,    # -N
  'wall_size'      => nil,    # -W  ( == geometry string :: "WxH")
  'color_depth'    => nil,    # -D
  'deferupdate'    => nil,    # -E
  'font_path'      => nil,    # -F
}

##################################################

opt = nil

set_opt = proc{|key, type|
  if type == ?f  # flag option -> with no value
    if opt =~ /^-.0$/ || opt =~ /^--without(_|-)/
      OPTION[key] = false
    else
      OPTION[key] = true
    end

  elsif type == ?!  # flag option -> with no value
    if opt =~ /^-.1$/ || opt =~ /^--with(_|-)/
      OPTION[key] = true
    else
      OPTION[key] = false
    end

  else
    if opt[0,2] == '--' || (val = opt[2..-1]).empty?
      OPTION[key] = ARGV.shift
    else
      OPTION[key] = val
    end

    case type
    when ?n  # numeric option
      OPTION[key] = Integer(OPTION[key])
    when ?s  # file-path option
      # do nothing
    end
  end
}

##################################################

while ARGV[0] =~ /^-/
  opt = ARGV.shift
  break if opt == '--'

  case opt
  when /^-A/, /^--xauth(_|-)dir$/
    set_opt.call('xautn_dir', ?s)

  when /^-a/, /^--xauth(_|-)path$/
    set_opt.call('xautn_path', ?s)

  when /^-b/, /^--ruby(_|-)bin$/
    set_opt.call('ruby_bin', ?s)

  when /^-C/, /^--control(_|-)passwd$/
    set_opt.call('control_passwd', ?s)

  when /^-c/, /^--control(_|-)port$/
    set_opt.call('control_port', ?n)

  when /^-D/, /^--color(_|-)depth$/
    set_opt.call('color_depth', ?n)

  when /^-d/, /^--debug(_|-)mode$/, /^--with(out)?(_|-)debug(_|-)mode$/
    set_opt.call('debug_mode', ?f)

  when /^-E/, /^--deferupdate$/
    set_opt.call('deferupdate', ?n)

  when /^-F/, /^--font(_|-)path$/
    set_opt.call('font_path', ?s)

  when /^-f/, /^--logfile$/
    set_opt.call('logfile', ?s)

  when /^-G/, /^--server(_|-)group$/
    set_opt.call('server_group', ?s)

  when /^-g/, /^--group$/
    set_opt.call('group', ?s)

  when /^-H/, /^--use(_|-)tmp(_|-)home$/, /^--with(out)?(_|-)use(_|-)tmp(_|-)home$/
    set_opt.call('use_tmp_home', ?f)

  when /^-h/, /^--hostaddr$/
    set_opt.call('hostaddr', ?s)

  when /^-I/, /^--interactive$/, /^--with(out)?(_|-)interactive$/
    set_opt.call('interactive', ?f)

  when /^-L/, /^--reverse(_|-)lookup$/, /^--with(out)?(_|-)reverse(_|-)lookup$/
    set_opt.call('reverse_lookup', ?f)

  when /^-l/, /^--listen$/
    set_opt.call('listen', ?n)

  when /^-N/, /^--desktop(_|-)name$/
    set_opt.call('desktop_name', ?s)

  when /^-n/, /^--null(_|-)device$/
    set_opt.call('null_device', ?s)

  when /^-o/, /^--tkorca(_|-)path$/
    set_opt.call('tkorca_path', ?s)

  when /^-p/, /^--port$/
    set_opt.call('port', ?n)

  when /^-R/, /^--rfbauth(_|-)passwd$/
    set_opt.call('rfbauth_passwd', ?s)

  when /^-r/, /^--app(_|-)root(_|-)dir$/
    set_opt.call('app_root_dir', ?s)

  when /^-S/, /^--hangup(_|-)signal$/
    set_opt.call('hangup_signal', ?s)

  when /^-s/, /^--serverrc$/
    set_opt.call('serverrc', ?s)

  when /^-T/, /^--tmp(_|-)home(_|-)dir$/
    set_opt.call('tmp_home_dir', ?s)

  when /^-t/, /^--tmp(_|-)dir$/
    set_opt.call('tmp_dir', ?s)

  when /^-U/, /^--server(_|-)user$/
    set_opt.call('server_user', ?s)

  when /^-u/, /^--user$/
    set_opt.call('user', ?s)

  when /^-v/, /^--version$/, /^--with(out)?(_|-)version$/
    set_opt.call('version', ?f)

  when /^-W/, /^--wall(_|-)size$/
    set_opt.call('wall_size', ?s)

  when /^-x/, /^--xvnc(_|-)path$/
    set_opt.call('xvnc_path', ?s)

  else
    raise RuntimeError, "unknown option: `#{opt}'"
  end
end
