#
#  tkorca_accepter.rb : 
#
#                             by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
require 'socket'
require 'thread'
require 'server/tkorca_service.rb'

module TkORCA_Accepter
  @gate = nil
  @accept_loop = nil
end

class << TkORCA_Accepter
  def start
    if @gate && ! @gate.closed?
      raise RuntimeError, "application service is already running"
    end
    create_gate()
    create_accept_loop_thread()
    self
  end

  def restart
    shutdown()
    create_gate() 
    create_accept_loop_thread()
    self
  end

  def shutdown
    (@gate.shutdown(Socket::SHUT_RDWR) if @gate && !@gate.closed?)  rescue nil
    @accept_loop.kill if @accept_loop  && @accept_loop.alive?
    @gate.close rescue nil
    self
  end

  #############################
  private
  #############################
  def create_gate
    begin
      @gate = TCPServer.open(SERVER_PARAM['hostaddr'], SERVER_PARAM['port'])
      @gate.listen(SERVER_PARAM['listen'])

    rescue Exception => e
      LOGGER.log("ERROR: #{e.class}: #{e.message}")
      LOGGER.log("fail to create a service port :", 
                 "host = `#{SERVER_PARAM['hostaddr']}',", 
                 "port = #{SERVER_PARAM['port']},", 
                 "listen = #{SERVER_PARAM['listen']}")
      raise e

    else
      LOGGER.log("create a service port :", 
                 "host = `#{SERVER_PARAM['hostaddr']}',", 
                 "port = #{SERVER_PARAM['port']},", 
                 "listen = #{SERVER_PARAM['listen']}")
    end
  end

  def is_acceptable?(sock)
    # This method may be overridden in server RC file.
    # If need to check peer's address, please override this.
    true
  end

  def create_accept_loop_thread
    @accept_loop = Thread.new{
      loop {
        sock = @gate.accept

        begin
          if is_acceptable?(sock)
            TkORCA_Service.new(sock)
          else
            LOGGER.log("Warning: reject connection #{sock.peeraddr.inspect}")
            sock.close rescue nil
          end
        rescue Exception => e
          LOGGER.log("ERROR: #{e.class}: #{e.message}")
          LOGGER.log("fail to start Ruby/TkORCA application service", 
                     "for #{sock.peeraddr.inspect}")
        end
      }
    }
  end
end
