#
#  xvnc_process.rb
#
#                             by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
require 'server/xvnc_options.rb'

module Xvnc_Process_Handler
  def exec_xvnc_process(display_num, xauth_file, sock, orig_mod_opts={})
    ENV['XAUTHORITY'] = xauth_file

    if SERVER_PARAM['xvnc_path']
      xvnc = SERVER_PARAM['xvnc_path']
    else
      xvnc = File.expand_path(DEFAULT_XVNC_PATH)
    end

    ps_name = nil
    if Object.const_defined?('XVNC_PROCESS_NAME')
      ps_name = XVNC_PROCESS_NAME
    elsif Object.const_defined?('DEFAULT_XVNC_PROCESS_NAME')
      ps_name = DEFAULT_XVNC_PROCESS_NAME
    end

    if ps_name
      exec_arg0 = [xvnc, ps_name]
      exec_arg0_str = "[#{xvnc}, #{ps_name}]"
    else
      exec_arg0 = xvnc
      exec_arg0_str = xvnc
    end

    options = []
    mod_opts = orig_mod_opts.dup

    # remove auth option for security reason (add `-auth xauth_file' later)
    mod_opts.delete('auth')

    # check Xvnc options
    DEFAULT_XVNC_OPTION_LIST.each{|k, v|
      val = (mod_opts.has_key?(k))? mod_opts.delete(k): v
      if val == true
        options << ('-' << k)
      elsif val
        options << ('-' << k) << val.to_s
      else
        # ignore
      end
    }

    # check unknown options 
    mod_opts.each{|k, v|
      next if DEFAULT_XSERVER_OPTION_LIST.has_key?(k)
      val = mod_opts.delete(k)
      if val == true
        options << ('-' << k)
      elsif val
        options << ('-' << k) << val.to_s
      else
        # ignore
      end
    }

    # check Xserver options
    DEFAULT_XSERVER_OPTION_LIST.each{|k, v|
      val = (mod_opts.has_key?(k))? mod_opts.delete(k): v
      if val == true
        options << ('-' << k)
      elsif val
        options << ('-' << k) << val.to_s
      else
        # ignore
      end
    }

    if USE_ID_CHANGE_PRIVILEGE
      Process::GID.change_privilege(SERVER_PARAM['group'])
      Process::UID.change_privilege(SERVER_PARAM['user'])
    end

    $stdin.reopen(sock)
    $stdout.reopen(sock)
    $stderr.reopen(sock) # stderr will be close on Xvnc

    # NOTE: Not work display_num parameter with '-inetd' option !!!
    #       So, have to believe the result of get_free_display_num().
    #       See `new_display_num.rb'.
    LOGGER.log("ID[#{@connection_id}]", 
               "exec #{[exec_arg0_str, *options].join(' ').inspect}")

    # close unnecessary I/O objects
    ObjectSpace.each_object{|obj|
      if obj.kind_of?(IO)
        unless obj == sock || obj == $stdin || obj == $stdout || obj == $stderr
          obj.close rescue nil
        end
      end
    }

    # Now, add xauthfile option ( to hide xauth filename from log )
    options << '-auth' << xauth_file

    exec(exec_arg0, *options)
  end

  def fork_xvnc(display_num, xauth_file, sock, mod_opts={})
    fork{ exec_xvnc_process(display_num, xauth_file, sock, mod_opts) }
  end
end
