#!/usr/bin/env ruby
#
#  sample class of handling I/O stream on a TkText widget
#                             by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
#  NOTE: TkTextIO supports 'character' (not 'byte') access only. 
#        So, for example, TkTextIO#getc returns a character, TkTextIO#pos 
#        means the character position, TkTextIO#read(size) counts by 
#        characters, and so on.
#        Of course, it is available to make TkTextIO class to suuport 
#        'byte' access. However, it may break multi-byte characters. 
#        and then, displayed string on the text widget may be garbled.
#        I think that it is not good on the supposed situation of using 
#        TkTextIO. 
#
require 'tk'
require 'thread'
require 'tkorca/tktextio'

module TkORCA
  class Console
    class ConsoleObj < TkTextIO
      OPT_DEFAULTS = {
        'mode'       => nil,
        'overwrite'  => false, 
        'text'       => nil, 
        'show'       => :pos, 
        'wrap'       => 'char', 
        'sync'       => true, 
        'prompt'     => nil, 
        'prompt_cmd' => nil, 
      }
    end

    module ConsoleAccess
      def has_console?
        console = @console[MultiTkIp.__getip]
        (console)? true: console
      end

      def close
        console = @console[MultiTkIp.__getip]
        if console
          begin
            @console.destroy
          rescue
          end
        end
        @console[MultiTkIp.__getip] = false
        self
      end

      def create(title=nil, keys=nil)
        if @console[MultiTkIp.__getip]
          fail RuntimeError, 'IP already has a console'
        end
        self.configure(title, keys)
      end

      def _create_sys_console(title, keys)
        ip = MultiTkIp.__getip
        wall = TkORCA::RFB_Canvas.get_wall
        @top[ip] = frame = TkORCA::Window_Frame.new(wall, title, 0, 0)
        frame.close_button_notify('Really close this console?')
        top = TkToplevel.new(:use=>frame.winid)
        @console[ip] = ConsoleObj.new(top) {
          configure(keys) if keys
          yscrollbar(TkScrollbar.new(top, :width=>10).pack(:side=>:right, 
                                                           :fill=>:y))
        }.pack(:expand=>true, :fill=>:both, :side=>:left)

        wall.bind_append('Destroy'){
          @top[ip].destroy rescue nil
          top.destroy rescue nil
        }

        @console[ip].bind('Destroy'){
          @top[ip].destroy rescue nil
          top.destroy rescue nil
        }
      end
      private :_create_sys_console

      def configure(title=nil, keys=nil)
        if title.kind_of?(Hash)
          keys = title
          title = nil
        elsif title
          title = title.to_s
        end
        if keys.kind_of?(Hash)
#          keys = _symbolkey2str(keys.dup)
          newkeys = {}
#          keys.each{|k,v| title << ([k.to_s,v].inspect)}
          keys.each{|k,v| newkeys[k.to_s] = v}
          keys = newkeys
          title = keys.delete('title') if keys.has_key?('title')
        elsif keys
          fail ArgumentError, 'expace a Hash to configure'
        end

        ip = MultiTkIp.__getip

        if (console = @console[ip])
          console.winfo_toplevel.title = title if title
          console.configure(keys) if keys
        elsif TkORCA.is_mother? # create new system console
          title = 'System Console' unless title
          _create_sys_console(title, keys)
        else # create new console
          title = 'Console' unless title
          @top[ip] = top = TkToplevel.new(:title=>title)
          ip._invoke('wm', 'close_button_notify', top.path, 
                     'Really close this console?')
          @console[ip] = ConsoleObj.new(top) {
            configure(keys) if keys
            yscrollbar(TkScrollbar.new(top, :width=>10).pack(:side=>:right, 
                                                             :fill=>:y))
          }.pack(:expand=>true, :fill=>:both, :side=>:left)

          @console[ip].bind('Destroy'){
            @console[ip] = nil
            @top[ip].destroy
          }
        end
        
        self
      end

      def destroy
        ip = MultiTkIp.__getip
        begin
          @console[ip].destroy if @console[ip]
        rescue
        end
      end

      def geometry(geo = nil)
        @top[MultiTkIp.__getip].geometry(geo)
      end

      def iconify
        @top[MultiTkIp.__getip].iconify
      end

      def deiconify
        @top[MultiTkIp.__getip].deiconify
      end

      def withdraw
        @top[MultiTkIp.__getip].withdraw
      end

      def raise(above=TkUtil::None)
        @top[MultiTkIp.__getip].raise(above)
      end

      def lower(below=TkUtil::None)
        @top[MultiTkIp.__getip].lower(below)
      end
    end
  end

  #############################################

  class Console
    @top = {}.taint
    @console = {}.taint

    ######################################
    # class & instance methods
    extend  ConsoleAccess
    include ConsoleAccess

    ######################################
    # class methods
    def self.method_missing(id, *args)
      if (console = @console[MultiTkIp.__getip])
        console.__send__(id, *args)
      else
        fail NoMethodError, "undefind method `#{id}'"
      end
    end

    def self.console_obj
      @console[MultiTkIp.__getip]
    end

    ######################################
    # instance methods

    def method_missing(id, *args)
      console = @console[MultiTkIp.__getip]
      if console == false                  # closed
        fail IOError, 'closed consoleinal'
      elsif console
        if console == true
          @default.__send__(id, *args)  # send to DEFAULT
        else
          console.__send__(id, *args)
        end
      else
        console = @class.has_console?
        if console
          # @class.__send__(id, *args)
          @class.console_obj.__send__(id, *args)
        elsif console == false          # closed
          fail IOError, 'closed console'
        else
          @default.__send__(id, *args)
        end
      end
    end

    def write(obj)
      method_missing(:write, obj)
    end

    def initialize(default_IO, title = nil)
      @console = {}.taint
      @default = default_IO
      @class = self.class
      # @class.configure(title)
    end
  end
end
