#
#  tkorca/daughter.rb : Ruby/TkORCA Daughter class
#
#                         by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#

require 'forwardable'

module TkORCA
  class Daughter
    extend Forwardable

    ###############################

    def inspect
      super() =~ /^(\S+)\s/
      $1 + '>'
    end

    ###############################

    def self._name(ip)
      ip.ip_name.gsub(/slave/, 'daughter')
    end

    ###############################

    def __setup_pseudo_toplevel
      th = Thread.new{
        Thread.stop unless Thread.current['IP']
        load(File.join(TkORCA::LIB_DIR, 'generate_pseudo_toplevel.rb'), true)
      }
      th['IP'] = @ip
      th.run.value

      if (top = @ip.__pseudo_toplevel) && (@ip.safe_level >= 4)
        top.taint
      end
    end
    private :__setup_pseudo_toplevel

    ###############################

    def initialize(trust, safe_level, winid, display=nil)
      keys = {:use=>winid}
      keys[:display] = display if display

      if trust
        # p ['trusted', safe_level]
        @ip = MultiTkIp.new_trusted_slave(safe_level, keys)
      else
        # p ['safe-Tk', safe_level]
        @ip = MultiTkIp.new_safeTk(safe_level, keys)
      end

      @loaded_scripts = []

      #---------------------------------------------

      @global_grab = [false]

      #---------------------------------------------

      __setup_pseudo_toplevel()

      #---------------------------------------------

      @ip.eval_proc{ TkORCA::OptionDB.setup  }

      @ip.set_cb_error{|e| _cb_error_proc(e) }
    end

    def_delegators(:@ip, :alive?, :safe?, :safe_base?, :safe_level, 
                   :ip_name, :slaves, :manipulable?, :_eval, :_invoke, 
                   :__pseudo_toplevel, :__pseudo_toplevel=, 
                   :__pseudo_toplevel_evaluable?, 
                   :__pseudo_toplevel_evaluable=)
    def_delegator(:@ip, :bg_eval_proc, :eval)
    def_delegator(:@ip, :path, :to_s)

    def __eval_proc__(*args, &blk)
      if TkORCA.is_mother?
        @ip.eval_proc(*args, &blk)
      end
    end

    def _cb_error_proc(err)
      msg = "#{err.class}: #{err.message}"
      msg << "\n\nfrom #{err.backtrace[0]}" if TkORCA::DEBUG.debug_mode
      Tk.messageBox(:icon=>'error', :type=>'ok', 
                    :title=>'Application Error', 
                    :message=>msg)
      '' # return an empty string
    end
    private :_cb_error_proc

#    alias __load__ load
#    private :__load__

#    def load_script(target_file, *mode)
    def load_script(target_file)
      TkORCA::LOG.log(TkORCA::Daughter._name(@ip), 
                      "load_script `#{target_file}'")
      @loaded_scripts << target_file
      # $0 = target_file
      $0.__set_progname_for_ip__(target_file, @ip)
      if $0 != target_file
        TkORCA::DEBUG.warn "WARNING!! : fail to set $0" + 
          " (the command line which launches me may be too short" + 
          " to keep the new value)"
      end
      @ip.bg_eval_proc{
        begin
          # __load__ target_file, true
          # @ip.__pseudo_toplevel.module_eval(IO.read(target_file),target_file)
          # @ip.__pseudo_toplevel.module_eval(IO.read(target_file), $0)
          #load(target_file, *mode)
          load(target_file)
        rescue Exception => e
          TkORCA::LOG.log(TkORCA::Daughter._name(@ip), 
                          "#{e.class.inspect}: #{e.message}")
          # TkORCA::DEBUG.p(e.inspect)
          TkORCA::DEBUG.write("#{e.class.inspect}: #{e.message}\n")
          e.backtrace.each{|s| TkORCA::DEBUG.write("\t#{s}\n")}
        end
      }
    end

    #############################

    def loaded_scripts
      @loaded_scripts
    end

    def current_script
      @loaded_scripts[-1]
    end
  end
end
