#
#  file_access.rb : utility for checking file access
#
#                           by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
module TkORCA::FileAccess
  @METHOD_TABLE = {}

  def self.method_missing(id, *args)
    unless (m = @METHOD_TABLE[id.to_s])
      fail NoMethodError, "undefined method `#{m}' for #{self.to_s}"
    end
    m.call(*args)
  end

  def self.register_method(id, cmd = Proc.new)
    TkORCA.security_check()
    @METHOD_TABLE[id.to_s] = cmd
    self
  end

  def self.alias_method(new_id, target_id)
    TkORCA.security_check()
    unless (cmd = @METHOD_TABLE[target_id.to_s])
      fail NameError, "undefined method `#{target_id.to_s}' for #{self.to_s}"
    end
    @METHOD_TABLE[new_id.to_s] = cmd
    self
  end

  #def self.get_table
  #  TkORCA.security_check()
  #  @METHOD_TABLE
  #end

  self.freeze
end

#####################################################
#  Each of followings is just an example.
#  In practical use, please define your restriction
#  by `TkORCA::FileAccess.register_method'
#  (e.g. define them on "mother_rcfile").
#####################################################

TkORCA::FileAccess.register_method(:allow_to_load?){|file, *args|
  # If acceptable, return pathname. And if not, return nil.
  File.expand_path(file)
}

TkORCA::FileAccess.register_method(:allow_to_require?){|file, *args|
  # If acceptable, return filename. And if not, return nil.
  file
}

TkORCA::FileAccess.register_method(:allow_tcl_to_open?){|file, *args|
  # If acceptable, return filename. And if not, return nil.
  return nil if file[0,1] == '|'  # command pipeline -> deny
  file
}

TkORCA::FileAccess.register_method(:allow_tcl_to_cd?){|dirname|
  # If acceptable, return dirpath. And if not, return nil.
  dirname
}

TkORCA::FileAccess.register_method(:check_glob_result?){|list, *glob_args|
  names = TclTkLib._split_tklist(list).collect{|n|
    n # dummy operation
  }
  TclTkLib._merge_tklist(*names)
}
