#
#  tkorca/image_defs.rb : images or bitmaps for Ruby/TkORCA
#
#                           by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#

module TkORCA
  module Images
    @table = {}

    def self.[](img_name)
      img_name = img_name.to_s
      @table[img_name] || (raise RuntimeError,"unknown image `#{img_name}'")
    end
  end
end

module TkORCA::Images
  ############################################
  #  null bitmap
  ############################################
  data = <<EOD
#define null_bmp_width 1
#define null_bmp_height 1
static unsigned char null_bmp_bits[] = {
   0x00};
EOD

  @table['null_bmp'] = TkBitmapImage.new(:data=>data)

  ############################################
  #  close button
  ############################################
  data = <<EOD
#define close_btn_width 16
#define close_btn_height 16
static unsigned char close_btn_bits[] = {
   0xf8, 0x3f, 0x04, 0x40, 0x02, 0x80, 0x1a, 0xb0, 0x32, 0x98, 0x62, 0x8c,
   0xc2, 0x86, 0x82, 0x83, 0x82, 0x83, 0xc2, 0x86, 0x62, 0x8c, 0x32, 0x98,
   0x1a, 0xb0, 0x02, 0x80, 0x04, 0x40, 0xf8, 0x3f};
EOD

  mask = <<EOD
#define close_btn_mask_width 16
#define close_btn_mask_height 16
static unsigned char close_btn_mask_bits[] = {
   0xff, 0xff, 0x07, 0xc0, 0x03, 0x80, 0x1b, 0xb0, 0x33, 0x98, 0x63, 0x8c,
   0xc3, 0x86, 0x83, 0x83, 0x83, 0x83, 0xc3, 0x86, 0x63, 0x8c, 0x33, 0x98,
   0x1b, 0xb0, 0x03, 0x80, 0x07, 0xc0, 0xff, 0xff};
EOD

  @table['close_btn'] = TkBitmapImage.new(:data=>data, :maskdata=>mask)

  ############################################

  data = <<EOD
#define minimize_btn_width 16
#define minimize_btn_height 16
static unsigned char minimize_btn_bits[] = {
   0xf8, 0x3f, 0x04, 0x40, 0x02, 0x80, 0x02, 0x80, 0x02, 0x80, 0x02, 0x80,
   0x02, 0x80, 0x02, 0x80, 0x02, 0x80, 0x02, 0x80, 0x02, 0x80, 0xf2, 0x9f,
   0xf2, 0x9f, 0x02, 0x80, 0x04, 0x40, 0xf8, 0x3f};
EOD

  mask = <<EOD
#define minimize_btn_mask_width 16
#define minimize_btn_mask_height 16
static unsigned char minimize_btn_mask_bits[] = {
   0xff, 0xff, 0x07, 0xc0, 0x03, 0x80, 0x03, 0x80, 0x03, 0x80, 0x03, 0x80,
   0x03, 0x80, 0x03, 0x80, 0x03, 0x80, 0x03, 0x80, 0x03, 0x80, 0xf3, 0x9f,
   0xf3, 0x9f, 0x03, 0x80, 0x07, 0xc0, 0xff, 0xff};
EOD
  @table['minimize_btn'] = TkBitmapImage.new(:data=>data, :maskdata=>mask)

  ############################################

  data = <<EOD
#define deiconify_btn_width 16
#define deiconify_btn_height 16
static unsigned char deiconify_btn_bits[] = {
   0xf8, 0x3f, 0x04, 0x40, 0x02, 0x80, 0xf2, 0x9f, 0xf2, 0x9f, 0x12, 0x90,
   0x12, 0x90, 0x12, 0x90, 0x12, 0x90, 0x12, 0x90, 0x12, 0x90, 0x12, 0x90,
   0xf2, 0x9f, 0x02, 0x80, 0x04, 0x40, 0xf8, 0x3f};
EOD

  mask = <<EOD
#define deiconify_mask_btn_width 16
#define deiconify_mask_btn_height 16
static unsigned char deiconify_mask_btn_bits[] = {
   0xff, 0xff, 0x07, 0xc0, 0x03, 0x80, 0xf3, 0x9f, 0xf3, 0x9f, 0x13, 0x90,
   0x13, 0x90, 0x13, 0x90, 0x13, 0x90, 0x13, 0x90, 0x13, 0x90, 0x13, 0x90,
   0xf3, 0x9f, 0x03, 0x80, 0x07, 0xc0, 0xff, 0xff};
EOD

  @table['deiconify_btn'] = TkBitmapImage.new(:data=>data, :maskdata=>mask)
end
