#
#  interactive_mode.rb : load IRB to control
#
#                           by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
require 'irb'

############################
#  constants
############################
module TkORCA
  IRB::TOPLEVEL_BINDING = @TOPLEVEL_BINDING
  IRB::ENV = @ENV
end

############################
#  get mother object
############################
module TkORCA
  mother = MOTHER
  @GET_MOTHER_PROC = proc{ (TkORCA.is_mother?)? mother: nil }

  def TkORCA.mother
    @GET_MOTHER_PROC.call
  end
end


############################
#  hack prompting of IRB
############################
class IRB::Context
  def verbose?
    false
  end

  def prompting?
    true
  end
end


############################
#  hide IRB from daughters
############################
m = Module.new
m.instance_variable_set('@mIRB', IRB)
m.instance_variable_set('@mIRB_undef', 
                        (class << IRB; method(:undef_method); end) )

class << m
  undef_method(:instance_variable_get, :instance_variable_set, :instance_eval)
  def const_missing(name)
    if TkORCA.is_mother?
      @mIRB.const_get(name)
    else
      super(name)
    end
  end
  def method_missing(id, *args)
    if TkORCA.is_mother?
      @mIRB.__send__(id, *args)
    else
      super(id, *args)
    end
  end
  def undef_method(*methods)
    @mIRB_undef.call(*methods)
  end
  def CurrentContext
    @mIRB.conf[:MAIN_CONTEXT]
  end
end

class Object
  remove_const 'IRB'
end
#IRB = TkORCA::IRB = m
::IRB = m

#
#  hack to avoid a trouble on `irb_exit'
#
module Kernel
  def irb_exit(irb_obj, ret = 0)
    MultiTkIp.exit
  end
end


############################
#  IRB thread
############################
module TkORCA
  unless self.const_defined?(:IRB_PREFIX)
    IRB_PREFIX = 'tkorca:'
  end
  unless self.const_defined?(:IRB_SHOW_LINE_NUM)
    IRB_SHOW_LINE_NUM = false
  end

  IRB_THREAD = Thread.new{
    begin
      IRB.init_config(nil)
      IRB.conf[:USE_READLINE] = false
      hist_num = (IRB_SHOW_LINE_NUM)? "%03n:": ""
      IRB.conf[:PROMPT][:TkORCA] = {
        :PROMPT_I => IRB_PREFIX + hist_num + "%i> ", 
        :PROMPT_N => IRB_PREFIX + hist_num + "%i> ", 
        :PROMPT_S => IRB_PREFIX + hist_num + "%i%l ", 
        :PROMPT_C => IRB_PREFIX + hist_num + "%i* ", 
        :RETURN => "=> %s\n"
      }
      IRB.conf[:PROMPT_MODE] = :TkORCA

      IRB.init_error
      irb = IRB::Irb.new
      IRB.conf[:MAIN_CONTEXT] = irb.context

      last_stat = nil
      Thread.current['SIGINT_trap'] = proc{ 
        stat = [
          ltype = irb.scanner.instance_variable_get('@ltype'), 
          continue = irb.scanner.instance_variable_get('@continue'), 
          indent = irb.scanner.instance_variable_get('@indent'),
          line_no = irb.scanner.instance_variable_get('@line_no')
        ]

        if last_stat == stat
          st = nil
        elsif ltype || continue || indent > 0
          st = true
        else
          st = false  # ignore SIGINT
        end
        last_stat = stat

        st
      }
      def (Thread.current['SIGINT_trap']).receive?
        self.call
      end

=begin
      class << IRB
        undef_method(:setup, :start, :init_config, :init_error, 
                     :parse_opts, :run_config, :load_modules, 
                     :rc_file, :rc_file_generators)
        self.freeze
      end
=end
      IRB.undef_method(:setup, :start, :init_config, :init_error, 
                       :parse_opts, :run_config, :load_modules, 
                       :rc_file, :rc_file_generators)

      class << IRB::Irb
        undef_method(:new)
        self.freeze
      end

      #class IRB::Irb
      IRB::Irb.class_eval{
        alias __eval_input__ eval_input
        private :__eval_input__

        def eval_input
          TkORCA.security_check(1)
          __eval_input__
        end
      }
      #end

      loop {
        catch(:IRB_EXIT){ irb.eval_input }
        $stdout.print "\n"
      }
    rescue => e
      STDERR.puts e.inspect
      STDERR.puts e.backtrace
    end
  }
end

####################################

def TkORCA.notify(msg, timeout=10)
  TkORCA.security_check()

  Thread.new{
    after_id = nil
    if timeout >= 0
      after_id = Tk.after((timeout * 1000).to_int){
        Tk.ip_eval('set ::tk::Priv(button) ok')
      }
    end

    Tk.messageBox(:icon=>'info', :message=>msg, :type=>'ok')

    Tk.after_cancel(after_id) if after_id
  }
end

def TkORCA.daughter_list
  TkORCA.security_check()

  list = {}
  TkORCA.mother.daughters.each{|d|
    list[d] = d.current_script
  }
  list
end

def TkORCA.kill_daughter(daughter)
  TkORCA.security_check()
  TkORCA.mother._remove_daughter(daughter)
  nil
end
