#
#  tkorca/opt_parse.rb : command line option parser
#
#    You have to make a hash `OPTION' which keeps option keys and values.
#
#                           by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
OPTION = {
  # flags
  'version'       => false,  # -v
  'sys_console'   => false,  # -c
  'debug_mode'    => false,  # -d
  'trust_mode'    => false,  # -D
  'disable_xim'   => false,  # -x
  'broadband'     => false,  # -B
  'interactive'   => false,  # -I
  'auth_check'    => false,  # -A

  # numeric
  'width'         => -1,     # -w
  'height'        => -1,     # -h
  'vroot_width'   => -1,     # -W
  'vroot_height'  => -1,     # -H
  'timeout'       => 300,    # -T
  'log_fileno'    => 2,      # -L  (default is STDERR)

  # string
  'hangup_signal' => nil,    # -S
  'log_prefix'    => nil,    # -P
  'root_dir'      => nil,    # -R
  'home_dir'      => nil,    # -r
  'tmp_dir'       => nil,    # -t
  'local_rcfile'  => nil,    # -l
  'mother_rcfile' => nil,    # -m
  'wall_image'    => nil,    # -i  (if `#xxxxxx', means `background color')
}

##################################################

opt = nil

set_opt = proc{|key, type|
  if type == ?f  # flag option -> with no value
    if opt =~ /^-.0$/ || opt =~ /^--without(_|-)/
      OPTION[key] = false
    else
      OPTION[key] = true
    end

  elsif type == ?!  # flag option -> with no value
    if opt =~ /^-.1$/ || opt =~ /^--with(_|-)/
      OPTION[key] = true
    else
      OPTION[key] = false
    end

  else
    if opt[0,2] == '--' || (val = opt[2..-1]).empty?
      OPTION[key] = ARGV.shift
    else
      OPTION[key] = val
    end

    case type
    when ?n  # numeric option
      OPTION[key] = Integer(OPTION[key])
    when ?s  # file-path option
      # do nothing
    end
  end
}

##################################################

while ARGV[0] =~ /^-/
  opt = ARGV.shift
  break if opt == '--'

  case opt
  when /^-B/, /^--broadband$/, /^--with(out)?(_|-)broadband$/
    set_opt.call('broadband', ?f)

  when /^-c/, /^--sys(_|-)console$/, /^--with(out)?(_|-)sys(_|-)console$/
    set_opt.call('sys_console', ?f)
    set_opt.call('interactive', ?!)  # excludsive for `--interactive'

  when /^-D/, /^--trust(_|-)mode$/, /^--with(out)?(_|-)trust_mode$/
    set_opt.call('debug_mode', ?f)
    set_opt.call('trust_mode', ?f)

  when /^-d/, /^--debug(_|-)mode$/, /^--with(out)?(_|-)debug_mode$/
    set_opt.call('debug_mode', ?f)

  when /^-A/, /^--auth(_|-)check$/, /^--with(out)?(_|-)auth(_|-)check$/
    set_opt.call('auth_check', ?f)

  when /^-H/, /^--vroot(_|-)height$/
    set_opt.call('vroot_height', ?n)

  when /^-h/, /^--height$/
    set_opt.call('height', ?n)

  when /^-I/, /^--interactive$/, /^--with(out)?(_|-)interactive$/
    set_opt.call('interactive', ?f)
    set_opt.call('sys_console', ?!)  # excludsive for `--sys_console'

  when /^-i/, /^--wall(_|-)image$/
    set_opt.call('wall_image', ?s)

  when /^-L/, /^--log(_|-)fileno$/
    set_opt.call('log_fileno', ?n)

  when /^-l/, /^--local(_|-)rcfile$/
    set_opt.call('local_rcfile', ?s)

  when /^-m/, /^--mother(_|-)rcfile$/
    set_opt.call('mother_rcfile', ?s)

  when /^-P/, /^--log(_|-)prefix$/
    set_opt.call('log_prefix', ?s)

  when /^-R/, /^--root(_|-)dir$/
    set_opt.call('root(_|-)dir', ?s)

  when /^-r/, /^--home(_|-)dir$/
    set_opt.call('home_dir', ?s)

  when /^-S/, /^--hangup(_|-)signal$/
    set_opt.call('hangup_signal', ?s)

  when /^-T/, /^--timeout$/
    set_opt.call('timeout', ?n)

  when /^-t/, /^--tmp(_|-)dir$/
    set_opt.call('tmp_dir', ?s)

  when /^-v/, /^--version$/, /^--with(out)?(_|-)version$/
    set_opt.call('version', ?f)

  when /^-W/, /^--vroot(_|-)width$/
    set_opt.call('vroot_width', ?n)

  when /^-w/, /^--width$/
    set_opt.call('width', ?n)

  when /^-x/, /^--disable(_|-)xim$/, /^--with(out)?(_|-)disable(_|-)xim$/
    set_opt.call('disable_xim', ?f)

  when /^--with(out)?(_|-)xim$/
    set_opt.call('disable_xim', ?!)

  else
    raise RuntimeError, "unknown option: `#{opt}'"
  end
end
