#
#  utils.rb : utility methods for Ruby/TkORCA
#
#                           by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#

module TkORCA
  @default_ip_id = MultiTkIp.__getip.object_id

  def self.is_mother?
    current = Thread.current

    if Thread.current.group != ThreadGroup::Default
      return (MultiTkIp.__getip.object_id == @default_ip_id)
    end

    ### current-threadgroup is default-group

    if current != TkORCA::EVLOOP_THREAD
      ### not callback operation --> mother
      return true
    end

    if current['callback_ip']
      ### current-interp is kept on thread-variable 'callback_ip'
      current['callback_ip'].object_id == @default_ip_id
    else
      MultiTkIp.__getip.object_id == @default_ip_id
    end
  end

  def self.security_check(safe_level = 4)
    unless is_mother?
      raise SecurityError, 'available on TkORCA mother thread only'
    end
    if $SAFE >= safe_level
      raise SecurityError, "cannot call when $SAFE >= #{safe_level}"
    end
  end
end
