#
#  window_frame.rb : 
#
#                           by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
require 'tk/canvas'
require 'tkorca/utils'
require 'tkorca/image_defs'
require 'tkorca/tkballoonhelp'

module TkORCA
  class Window_Frame < TkcWindow
    # @@coords = [200, 150]
    @@coords_delta  = [45, 30]     # [x, y]
    @@coords_seq    = [10, 160]    # [cnt, delta_x]
    @@coords_offset = [-40, 10, 2] # [x0, y0, init_num]

    # @@broadband_network = false

    @@frames = {}
    @@frame_table = {}

    ###############################

    def inspect
      super() =~ /^(\S+)\s/
      $1 + '>'
    end

    ###############################

    def self.get_frame(container)
      TkORCA.security_check(5)

      return nil unless container

      if container.kind_of?(TkWindow)
        @@frame_table[container.path]
      else
        @@frame_table[container.to_s]
      end
    end

    def self.get_container_id(container_path)
      if (container = @@frame_table[container_path])
        container.id.to_s
      else
        ""
      end
    end

    ###############################

    def _get_frame_position
      d, m = (@@frames.size + @@coords_offset[2] - 1).divmod(@@coords_seq[0])
      [ 
        @wall.canvasx(0) + @@coords_offset[0] \
                               + d * @@coords_seq[1] + m * @@coords_delta[0], 
        @wall.canvasy(0) + @@coords_offset[1] + m * @@coords_delta[1]
      ]
    end
    private :_get_frame_position

    def _check_size_of_window(req_w, req_h)
      width, height = req_w, req_h
      min_x, min_y, max_x, max_y = @size_limit

      # keep defined minimum width
      min_x = @titlebar_reqwidth if min_x < @titlebar_reqwidth

      # minsize has higher priority than maxsize
      width = max_x if width > max_x
      width = min_x if width < min_x

      height = max_y if height > max_y
      height = min_y if height < min_y

      # aspect check
      min_numer, min_denom, max_numer, max_denom = @aspect

      if (param_w = width * max_denom) > (param_h = height * max_numer)
        if (param_w = min_x * max_denom) > param_h
          width = min_x
          if max_y > min_y
            height = (param_w > max_y * max_numer)? max_y: param_w / max_numer
          end
        else
          width = param_h / max_denom
        end
      end

      if (param_w = width * min_denom) < (param_h = height * min_numer)
        if param_w < (param_h = min_y * min_numer)
          height = min_y
          if max_x > min_x
            width = (max_x * min_denom < param_h)? max_x: param_h / min_denom
          end
        else
          height = param_w / min_numer
        end
      end

      [width, height]
    end
    private :_check_size_of_window

    def _title_bind
      @rx = 0; @ry = 0; @sx = 0; @sy = 0

      @titlebar.bind('Enter',  proc{|w, x, y| @base.cursor = ''}, '%W %x %y')
      @titlebar.bind('Motion', proc{|w, x, y| @base.cursor = ''}, '%W %x %y')

      @titlebar.bind('Shift-ButtonPress-1', 
                     proc{|rx, ry| 
                       @base.cursor = ''
                       @titlebar.cursor = 'fleur'
                       @rx = rx; @ry = ry; @sx, @sy = self.coords
                     }, '%X', '%Y')

      @titlebar.bind('ButtonPress-1', 
                     proc{|rx, ry| 
                       @base.cursor = ''
                       @titlebar.cursor = 'fleur'
                       @rx = rx; @ry = ry; @sx, @sy = self.coords
                       self.raise
                     }, '%X', '%Y')

      @titlebar.bind('ButtonRelease-1', 
                     proc{|rx, ry| 
                       #_hide_resize_mask('... resize ...')
                       _hide_resize_mask('')
                       @titlebar.cursor = ''
                     }, '%X', '%Y')

      @titlebar.bind('ButtonPress-3', 
                     proc{|rx, ry| 
                       @base.cursor = ''
                       @rx = rx; @ry = ry; @sx, @sy = self.coords
                       self.lower
                     }, '%X', '%Y')

      @titlebar.bind('B1-Motion', 
                     proc{|rx, ry|
                       # @base.cursor = ''
                       wx = @wall.winfo_rootx; wy = @wall.winfo_rooty
                       if rx > wx && rx < wx + @wall.winfo_width &&
                           ry > wy && ry < wy + @wall.winfo_height
                         x = @sx + (rx - @rx)
                         y = @sy + (ry - @ry)
                         self.coords = [x, y]
                         x1, y1, x2, y2 = self.bbox
                         unless @iconic
                           _show_resize_mask(x1 + @container.winfo_x, 
                                             y1 + @container.winfo_y, 
                                             :nw, ((@show_resize_mask)? \
                                                   nil: '... move ...'))
                         end
                       end
                     }, '%X', '%Y')
    end
    private :_title_bind

    def _set_edge_cursor(w, x, y)
      return if @resizing[0] || @resizing[1] || @iconic
      current_cursor = w.cursor

      # p [w.path,x,y, @corner[:e], @corner[:s]]
      if x < @border
        if y <= @corner[:n]
          cursor = 'top_left_corner'
          btag = :top_left
        elsif y>= @corner[:s]
          cursor = 'bottom_left_corner'
          btag = :bottom_left
        else
          cursor = 'left_side'
          btag = :left
        end
      elsif x >= @width - @border
        if y <= @corner[:n]
          cursor = 'top_right_corner'
          btag = :top_right
        elsif y>= @corner[:s]
          cursor = 'bottom_right_corner'
          btag = :bottom_right
        else
          cursor = 'right_side'
          btag = :right
        end
      elsif y < @border
        if x <= @corner[:w]
          cursor = 'top_left_corner'
          btag = :top_left
        elsif x>= @corner[:e]
          cursor = 'top_right_corner'
          btag = :top_right
        else
          cursor = 'top_side'
          btag = :top
        end
      elsif y >= @height - @border
        if x <= @corner[:w]
          cursor = 'bottom_left_corner'
          btag = :bottom_left
        elsif x>= @corner[:e]
          cursor = 'bottom_right_corner'
          btag = :bottom_right
        else
          cursor = 'bottom_side'
          btag = :bottom
        end
      else
        cursor = ''
        btag = :none
      end

      @base.bindtags_shift
      @base.bindtags_unshift(@base_btags[btag])

      w.cursor = cursor if cursor != current_cursor
    end
    private :_set_edge_cursor

    def _set_resize_frame_color
      @base.background(@active_fg)
      @titlebar.background(@active_fg)
      @close_btn.background(@active_fg)
      @minimize_btn.background(@active_fg)
    end

    def _reset_resize_frame_color
      @base.background(@bg_color)
      @titlebar.background(@bg_color)
      @close_btn.background(@bg_color)
      @minimize_btn.background(@bg_color)
    end
    private :_set_resize_frame_color, :_reset_resize_frame_color

    def _show_resize_mask(x, y, anchor, text=nil)
      @resize_mask.coords(x, y)
      @resize_mask.configure(:anchor=>anchor, 
                             :width=>@container.winfo_width, 
                             :height=>@container.winfo_height)
      @show_resize_mask = true
      @resize_label.text = text if text
      #if @@broadband_network
      if @wall.broadband_mode?
        @resize_frame.lower
      else
        # @resize_mask.raise(self)
        @resize_frame.raise(@base)
      end
    end

    def _hide_resize_mask(text = nil)
      @resize_mask.configure(:anchor=>:se, :width=>0, :height=>0)
      @resize_mask.coords(-1, -1)
      @resize_mask.lower
      @resize_label.text = text if text
      @show_resize_mask = false
      @resizing[0] = @resizing[1] = false
    end
    private :_show_resize_mask, :_hide_resize_mask

    def _setup_bindtag_SW_anchor(btag, resizable_w, resizable_h)
      btag.bind('ButtonPress-1', proc{|x, y, rx, ry|
                  Tk.callback_break if @iconic

                  @resizing[0] = resizable_w && @resizable[0]
                  @resizing[1] = resizable_h && @resizable[1]

                  _set_resize_frame_color

                  Tk.callback_break unless @resizing[0] || @resizing[1]

                  # get and reset the size of the container before resiziing
                  @resize_base_w = @container.width  = @container.winfo_width
                  @resize_base_h = @container.height = @container.winfo_height

                  @base.pack_propagate(true) # if @geo_def
                  @saved_propagate = true

                  x1, y1, x2, y2 = self.bbox

                  @resize_base_x = rx - @resize_base_w
                  @resize_base_y = ry + @resize_base_h
                  self.coords(x1, y2)
                  self.anchor(:sw)
                  _show_resize_mask(x1 + @container.winfo_x, 
                                    y1 + @container.winfo_y + @resize_base_h, 
                                    :sw, '... resize ...')
                }, '%x %y %X %Y')
    end

    def _setup_bindtag_SE_anchor(btag, resizable_w, resizable_h)
      btag.bind('ButtonPress-1', proc{|x, y, rx, ry|
                  Tk.callback_break if @iconic

                  @resizing[0] = resizable_w && @resizable[0]
                  @resizing[1] = resizable_h && @resizable[1]

                  _set_resize_frame_color

                  Tk.callback_break unless @resizing[0] || @resizing[1]

                  # get and reset the size of the container before resiziing
                  @resize_base_w = @container.width  = @container.winfo_width
                  @resize_base_h = @container.height = @container.winfo_height

                  @base.pack_propagate(true) # if @geo_def
                  @saved_propagate = true

                  x1, y1, x2, y2 = self.bbox

                  @resize_base_x = rx + @resize_base_w
                  @resize_base_y = ry + @resize_base_h
                  self.coords(x2, y2)
                  self.anchor(:se)
                  _show_resize_mask(x1 + @container.winfo_x + @resize_base_w, 
                                    y1 + @container.winfo_y + @resize_base_h, 
                                    :se, '... resize ...')
                }, '%x %y %X %Y')
    end
    private :_setup_bindtag_SW_anchor, :_setup_bindtag_SE_anchor

    def _setup_bindtag_NW_anchor(btag, resizable_w, resizable_h)
      btag.bind('ButtonPress-1', proc{|x, y, rx, ry|
                  Tk.callback_break if @iconic

                  @resizing[0] = resizable_w && @resizable[0]
                  @resizing[1] = resizable_h && @resizable[1]

                  _set_resize_frame_color

                  Tk.callback_break unless @resizing[0] || @resizing[1]

                  # get and reset the size of the container before resiziing
                  @resize_base_w = @container.width  = @container.winfo_width
                  @resize_base_h = @container.height = @container.winfo_height

                  @base.pack_propagate(true) # if @geo_def
                  @saved_propagate = true

                  x1, y1, x2, y2 = self.bbox

                  @resize_base_x = rx - @resize_base_w
                  @resize_base_y = ry - @resize_base_h
                  self.coords(x1, y1)
                  self.anchor(:nw)
                  _show_resize_mask(x1 + @container.winfo_x, 
                                    y1 + @container.winfo_y, 
                                    :nw, '... resize ...')
                }, '%x %y %X %Y')
    end

    def _setup_bindtag_NE_anchor(btag, resizable_w, resizable_h)
      btag.bind('ButtonPress-1', proc{|x, y, rx, ry|
                  Tk.callback_break if @iconic

                  @resizing[0] = resizable_w && @resizable[0]
                  @resizing[1] = resizable_h && @resizable[1]

                  _set_resize_frame_color

                  Tk.callback_break unless @resizing[0] || @resizing[1]

                  # get and reset the size of the container before resiziing
                  @resize_base_w = @container.width  = @container.winfo_width
                  @resize_base_h = @container.height = @container.winfo_height

                  @base.pack_propagate(true) # if @geo_def
                  @saved_propagate = true

                  x1, y1, x2, y2 = self.bbox

                  @resize_base_x = rx + @resize_base_w
                  @resize_base_y = ry - @resize_base_h
                  self.coords(x2, y1)
                  self.anchor(:ne)
                  _show_resize_mask(x1 + @container.winfo_x + @resize_base_w, 
                                    y1 + @container.winfo_y, 
                                    :ne, '... resize ...')
                }, '%x %y %X %Y')
    end
    private :_setup_bindtag_NW_anchor, :_setup_bindtag_NE_anchor

    def _setup_bindtag_top
      btag = TkBindTag.new
      _setup_bindtag_SE_anchor(btag, false, true)
      btag.bind('B1-Motion', proc{|x, y, rx, ry|
                  Tk.callback_break unless @resizing[1]
                  width, height = _check_size_of_window(@resize_base_w, 
                                                        @resize_base_y - ry)
                  @container.width  = @resize_mask.width  = width
                  @container.height = @resize_mask.height = height
                  @size_from = :user
                }, '%x %y %X %Y')
      btag
    end

    def _setup_bindtag_bottom
      btag = TkBindTag.new
      _setup_bindtag_NW_anchor(btag, false, true)
      btag.bind('B1-Motion', proc{|x, y, rx, ry|
                  Tk.callback_break unless @resizing[1]
                  width, height = _check_size_of_window(@resize_base_w, 
                                                        ry - @resize_base_y)
                  @container.width  = @resize_mask.width  = width
                  @container.height = @resize_mask.height = height
                  @size_from = :user
                }, '%x %y %X %Y')
      btag
    end

    def _setup_bindtag_left
      btag = TkBindTag.new
      _setup_bindtag_SE_anchor(btag, true, false)
      btag.bind('B1-Motion', proc{|x, y, rx, ry|
                  Tk.callback_break unless @resizing[0]
                  width, height = _check_size_of_window(@resize_base_x - rx, 
                                                        @resize_base_h)
                  @container.width  = @resize_mask.width  = width
                  @container.height = @resize_mask.height = height
                  @size_from = :user
                }, '%x %y %X %Y')
      btag
    end

    def _setup_bindtag_right
      btag = TkBindTag.new
      _setup_bindtag_NW_anchor(btag, true, false)
      btag.bind('B1-Motion', proc{|x, y, rx, ry|
                  Tk.callback_break unless @resizing[0]
                  width, height = _check_size_of_window(rx - @resize_base_x, 
                                                        @resize_base_h)
                  @container.width  = @resize_mask.width  = width
                  @container.height = @resize_mask.height = height
                  @size_from = :user
                }, '%x %y %X %Y')
      btag
    end

    def _setup_bindtag_top_left
      btag = TkBindTag.new
      _setup_bindtag_SE_anchor(btag, true, true)
      btag.bind('B1-Motion', proc{|x, y, rx, ry|
                  Tk.callback_break unless @resizing[0] || @resizing[1]
                  w = (@resizing[0])? @resize_base_x - rx : @resize_base_w
                  h = (@resizing[1])? @resize_base_y - ry : @resize_base_h
                  width, height = _check_size_of_window(w, h)
                  @container.width  = @resize_mask.width  = width
                  @container.height = @resize_mask.height = height
                  @size_from = :user
                }, '%x %y %X %Y')
      btag
    end

    def _setup_bindtag_top_right
      btag = TkBindTag.new
      _setup_bindtag_SW_anchor(btag, true, true)
      btag.bind('B1-Motion', proc{|x, y, rx, ry|
                  Tk.callback_break unless @resizing[0] || @resizing[1]
                  w = (@resizing[0])? rx - @resize_base_x : @resize_base_w
                  h = (@resizing[1])? @resize_base_y - ry : @resize_base_h
                  width, height = _check_size_of_window(w, h)
                  @container.width  = @resize_mask.width  = width
                  @container.height = @resize_mask.height = height
                  @size_from = :user
                }, '%x %y %X %Y')
      btag
    end

    def _setup_bindtag_bottom_left
      btag = TkBindTag.new
      _setup_bindtag_NE_anchor(btag, true, true)
      btag.bind('B1-Motion', proc{|x, y, rx, ry|
                  Tk.callback_break unless @resizing[0] || @resizing[1]
                  w = (@resizing[0])? @resize_base_x - rx : @resize_base_w
                  h = (@resizing[1])? ry - @resize_base_y : @resize_base_h
                  width, height = _check_size_of_window(w, h)
                  @container.width  = @resize_mask.width  = width
                  @container.height = @resize_mask.height = height
                  @size_from = :user
                }, '%x %y %X %Y')
      btag
    end

    def _setup_bindtag_bottom_right
      btag = TkBindTag.new
      _setup_bindtag_NW_anchor(btag, true, true)
      btag.bind('B1-Motion', proc{|x, y, rx, ry|
                  Tk.callback_break unless @resizing[0] || @resizing[1]
                  w = (@resizing[0])? rx - @resize_base_x : @resize_base_w
                  h = (@resizing[1])? ry - @resize_base_y : @resize_base_h
                  width, height = _check_size_of_window(w, h)
                  @container.width  = @resize_mask.width  = width
                  @container.height = @resize_mask.height = height
                  @size_from = :user
                }, '%x %y %X %Y')
      btag
    end

    private :_setup_bindtag_top, :_setup_bindtag_bottom, 
            :_setup_bindtag_left, :_setup_bindtag_right, 
            :_setup_bindtag_top_left, :_setup_bindtag_top_right, 
            :_setup_bindtag_bottom_left, :_setup_bindtag_bottom_right

    def _setup_base_bindtags
      @base_btags = {
        :top=>_setup_bindtag_top, 
        :bottom=>_setup_bindtag_bottom, 
        :left=>_setup_bindtag_left, 
        :right=>_setup_bindtag_right, 
        :top_left=>_setup_bindtag_top_left, 
        :top_right=>_setup_bindtag_top_right, 
        :bottom_left=>_setup_bindtag_bottom_left, 
        :bottom_right=>_setup_bindtag_bottom_right, 
        :none=>TkBindTag.new
      }

      @base.bindtags_unshift(@base_btags[:none])

      @base.bind('ButtonRelease-1', proc{|w, x, y, rx, ry|
                   _reset_resize_frame_color
                   Tk.callback_break unless @resizing[0] || @resizing[1]
                   _hide_resize_mask
                   @resizing[0] = @resizing[1] = false
                   if @geo_def || @size_from == :user
                     @base.pack_propagate(false)
                     @saved_propagate = false
                   end
                   _set_edge_cursor(w, x, y)
                 }, '%W %x %y %X %Y')
    end
    private :_setup_base_bindtags

    #############################################

    def base_frame
      @base
    end
    protected :base_frame

    #############################################

    def initialize(wall, title, *args)
      TkORCA.security_check(5)

      @wall = wall

      @tag = TkcNamedTag.new(@wall, 'window_frame_tag')

      @border = 3
      @fg_color = 'white'
      @bg_color = 'midnight blue'
      @active_fg = 'blue'

      # @base = TkFrame.new(@wall, :borderwidth=>@border, :relief=>:ridge)
      # @base = TkFrame.new(@wall, :borderwidth=>@border, 
      #                     :background=>@bg_color, :relief=>:flat)
      @base = TkFrame.new(@wall, :class=>self.class.to_s,
                          :takefocus=>false, :background=>@bg_color, 
                          :borderwidth=>@border, :relief=>:ridge)

      @base_rootx = TkWinfo.rootx(@base)
      @base_rooty = TkWinfo.rooty(@base)

      @width  = 1
      @height = 1

      @geo_def  = false
      @geo_mode = [?+, ?+]

      @resizable = [true,  true ]
      @resizing  = [false, false]

      @aspect = [0, 0, 0, 0]

      @size_from = nil  # nil / :program / :user
      @pos_from  = nil  # nil / :program / :user

      @wm_state = 'normal'

      @size_limit = [0, 0, 10000, 10000] # [ min_x, min_y, max_x, max_y ]
      @iconify_width = 50

      @corner_delta = 15
      @corner = {:n=>0, :w=>0, :s=>1, :e=>1}

      @base.bind('Configure', proc{|w,h|
                   @base_rootx = TkWinfo.rootx(@base)
                   @base_rooty = TkWinfo.rooty(@base)

                   @width = w
                   @height = h

                   c = (w-1)/2
                   @corner[:w] = (@corner_delta < c)? @corner_delta: c

                   c = (w+1)/2
                   @corner[:e] = (@corner_delta < w-c)? w-@corner_delta: c

                   c = (h-1)/2
                   @corner[:n] = (@corner_delta < c)? @corner_delta: c

                   c = (h+1)/2
                   @corner[:s] = (@corner_delta < h-c)? h-@corner_delta: c

                   #p [@width, @height, @corner, @base_rootx, @base_rooty]
                 }, '%w %h')

      @cursor_timer = TkTimer.new(50, -1, proc{
                                    p_x, p_y = TkWinfo.pointerxy(@base)
                                    x = p_x - @base_rootx
                                    y = p_y - @base_rooty
                                    _set_edge_cursor(@base, x, y)
                                  })

      @base.bind('Leave', proc{|w, x, y|
                   # p 'LEAVE'
                   @cursor_timer.stop
                   _set_edge_cursor(w, x, y)
                 }, '%W %x %y')

      @base.bind('Enter', proc{|w, x, y|
                   # p 'Enter'
                   @cursor_timer.restart(0)
                   _set_edge_cursor(w, x, y)
                 }, '%W %x %y')

      @base.bind('Motion', proc{|w, x, y|
                   # @cursor_timer.restart
                   _set_edge_cursor(w, x, y)
                 }, '%W %x %y')

      @resize_frame = TkFrame.new(@wall, :takefocus=>false, :borderwidth=>0)
      @resize_frame.pack_propagate(false)
      #@resize_label = TkLabel.new(@resize_frame, 
      #                            :takefocus=>false, :anchor=>:center, 
      #                            :text=>'... resize ...').pack(:fill=>:both, 
      #                                                          :expand=>true)
      @resize_label = TkLabel.new(@resize_frame, 
                                  :takefocus=>false, :anchor=>:center, 
                                  :text=>'').pack(:fill=>:both, :expand=>true)
      @resize_mask = TkcWindow.new(@wall, -1, -1, :anchor=>:se, 
                                   :window=>@resize_frame, 
                                   :width=>0, :height=>0)
      @show_resize_maskk = false

      _setup_base_bindtags

      ##############################

      @WM_PROTOCOLS = {}

      @titlebar = TkLabel.new(@base, :text=>" #{title} ", :takefocus=>false, 
                              :compound=>:left, :image=>Images['null_bmp'],
                              :width=>1, 
                              :padx=>0, 
                              :pady=>2, 
                              #:relief=>:raised, 
                              :foreground=>@fg_color, 
                              :background=>@bg_color).pack(:fill=>:x, :padx=>0)
      @titlebar_reqwidth = @titlebar.winfo_reqwidth

      Tk.update

      # fix size (height, width) to avoid flicker when showing balloon
      @titlebar.pack_propagate(false)

      Images['close_btn'].configure(:foreground=>@fg_color,
                                    :background=>@bg_color)

      @close_btn = TkButton.new(@titlebar, 
                                :image=>Images['close_btn'], 
                                :anchor=>:nw,
                                :padx=>0, :pady=>0,
                                :foreground=>@fg_color, 
                                :background=>@bg_color, 
                                #:activeforeground=>@fg_color, 
                                :activebackground=>@active_fg, 
                                :highlightthickness=>0, 
                                :relief=>:flat, 
                                :borderwidth=>0, 
                                :command=>proc{
                                  self.close
                                }).pack(:side=>:right, 
                                        :ipadx=>0, :ipady=>0, 
                                        :padx=>0, :pady=>0)

      @close_btn_notify = nil
      @btag_close = TkBindTag.new
      @btag_close.bind('ButtonRelease-1', '%W'){|btn|
        if @close_btn_notify
          ret = Tk.messageBox(:icon=>'warning', :type=>'okcancel', 
                              :default=>'cancel', :message=>@close_btn_notify)
        else
          ret = 'ok'
        end
        btn.invoke if ret == 'ok'
      }
      @close_btn.bindtags_unshift(@btag_close)

      Images['minimize_btn'].configure(:foreground=>@fg_color, 
                                       :background=>@bg_color) 
      Images['deiconify_btn'].configure(:foreground=>@fg_color,
                                        :background=>@bg_color)
      @iconic = false

      @minimize_btn = TkButton.new(@titlebar, 
                                   :image=>Images['minimize_btn'], 
                                   :anchor=>:nw,
                                   :padx=>0, :pady=>0,
                                   :foreground=>@fg_color, 
                                   :background=>@bg_color, 
                                   #:activeforeground=>@fg_color, 
                                   :activebackground=>@active_fg, 
                                   :highlightthickness=>0, 
                                   :relief=>:flat, 
                                   :borderwidth=>0, 
                                   :command=>proc{
                                     if @iconic
                                       self.deiconify
                                     else
                                       self.iconify
                                     end
                                   }).pack(:side=>:right, 
                                           :ipadx=>0, :ipady=>0, 
                                           :padx=>0, :pady=>0)

      @balloon = TkBalloonHelp.new(@titlebar, :interval=>800, 
                                   :relief=>:solid, :borderwidth=>1, 
                                   :background=>'white', 
                                   :wraplength=>100, :justify=>:left, 
                                   :command=>proc{|x, y, bhelp|
                                     bhelp.text = @titlebar.text
                                   })
      @balloon.disable

      @container = TkFrame.new(@base, :takefocus=>false, 
                               :container=>true, :borderwidth=>0,
                               :padx=>0, :pady=>0).pack(:fill=>:both,
                                                        :expand=>true)
      @container.bind('Configure', proc{|w,h|
                        # @container.width, @container.height = 
                        #   _check_size_of_window(w, h)
                        ww, hh = _check_size_of_window(w, h)
                        if ww != w || hh != h
                          # need to set both 
                          @container.width  = ww
                          @container.height = hh 
                        end
                      }, '%w %h')

      @container.bind('Enter',  proc{|w, x, y| @base.cursor = ''}, '%W %x %y')
      @container.bind('Motion', proc{|w, x, y| @base.cursor = ''}, '%W %x %y')

      # for proper work of 'overrideredirect' + 'withdraw'
      @dummy_frame = TkFrame.new(@base, :takefocus=>false, 
                                 :background=>@bg_color, 
                                 :width=>@iconify_width, :height=>0, 
                                 :borderwidth=>0, :highlightthickness=>0)

      @dummy_frame.bind('Enter', proc{|w, x, y| @base.cursor = ''}, '%W %x %y')
      @dummy_frame.bind('Motion',proc{|w, x, y| @base.cursor = ''}, '%W %x %y')


      @@frames[@base.path] = self
      @@frame_table[@container.path] = self

      @container.bind('Destroy'){
        @base.cursor = ''
        @@frames.delete(@base.path)
        @@frame_table.delete(@container.path)
        self.destroy
      }

      if args[0].kind_of?(Numeric) && args[1].kind_of?(Numeric)
        super(wall, *args)
      else
        x, y = _get_frame_position
        super(wall, x, y, *args)
      end

      self.addtag(@tag)

      self.window = @base
      self.anchor = :nw

      _title_bind
    end

    #############################################

    def close_button_notify(str)
      str = str.to_s if str
      @close_btn_notify = str
    end

    def winid
      TkORCA.security_check(5)

      @container.winfo_id
    end

    def title
      @titlebar.text
    end
    def title=(str)
      TkORCA.security_check(5)

      @titlebar.text(str)
    end

    def frame
      (@base.borderwidth == 0)? nil: winid()
    end

    def get_aspect
      @aspect.dup
    end
    def set_aspect(*args) # min_numer, min_denom, max_numer, max_denom
      fail ArgumentError, 'wrong number of arguments' if args.size != 4
      args.each{|arg|
        fail TypeError, 'expect Fixnum arguments' unless arg.kind_of?(Fixnum)
      }
      @aspect.replace(args)
      self
    end
    def unset_aspect
      set_aspect(0, 0, 0, 0)
    end

    def winfo_toplevel_geometry
      # base_x = @wall.winfo_rootx - @wall.canvasx(0).to_int
      # base_y = @wall.winfo_rooty - @wall.canvasy(0).to_int
      base_x = -(@wall.canvasx(0).to_int)
      base_y = -(@wall.canvasy(0).to_int)

      bx1, by1, bx2, by2 = self.bbox

      if @container.winfo_mapped?
        w = @container.winfo_width
        h = @container.winfo_height
      else
        w = @container.winfo_reqwidth
        h = @container.winfo_reqheight
      end

      format('%dx%d+%d+%d', w, h, 
             base_x + bx1 + @container.winfo_x, 
             base_y + by1 + @container.winfo_y)
    end

    def geometry(geo = nil)
      # minsize and maxsize have higher priority than geometry
      #base_x = @wall.winfo_rootx - @wall.canvasx(0).to_int
      #base_y = @wall.winfo_rooty - @wall.canvasy(0).to_int
      base_x = -(@wall.canvasx(0).to_int)
      base_y = -(@wall.canvasy(0).to_int)

      unless geo
        ### get geometry

        bx1, by1, bx2, by2 = self.bbox

        x = (@geo_mode[0] == ?+)? \
                (base_x + bx1): @wall.winfo_screenwidth - (base_x + bx2)
        y = (@geo_mode[1] == ?+)? \
                (base_y + by1): @wall.winfo_screenheight - (base_y + by2)

        if @container.winfo_mapped?
          w = @container.winfo_width
          h = @container.winfo_height
        else
          w = @container.winfo_reqwidth
          h = @container.winfo_reqheight
        end

        format('%dx%d%c%d%c%d', w, h, @geo_mode[0], x, @geo_mode[1], y)

      else
        ### set geometry
        unless geo =~ /^((\d+)x(\d+))?(([+-])(-?\d+)([+-])(-?\d+))?$/
          fail ArgumentError, "bad geometry specifier `#{geo}'"
        end
        unless (set_size = $1) || (set_pos = $4)
          @geo_def = false
          @base.pack_propagate(true)
          @saved_propagate = true
          return
        end
        if set_size
          width  = Integer($2)
          height = Integer($3)
        end
        if set_pos
          mode_x   = $5[0]
          offset_x = Integer($6)
          mode_y   = $7[0]
          offset_y = Integer($8)
        end

        # position
        if set_pos
          rx1, ry1, rx2, ry2 = @wall.scrollregion
          anchor_pos = ''

          if mode_y == ?+
            yy = offset_y - base_y
            anchor_pos += 'n'
          else
            yy = @wall.winfo_screenheight - offset_y - base_y
            anchor_pos += 's'
          end
          if mode_x == ?+
            xx = offset_x - base_x
            anchor_pos += 'w'
          else
            xx = @wall.winfo_screenwidth - offset_x - base_x
            anchor_pos += 'e'
          end

          self.coords(xx, yy)
          self.anchor(anchor_pos)

          position_from(:user)
        end

        # size
        if set_size
          w, h = _check_size_of_window(width, height)

          @base.pack_propagate(true)
          @saved_propagate = true

          @container.width  = w
          @container.height = h

          @geo_def = true

          size_from(:user)
        end

        self
      end
    end

    def get_maxsize
      @size_limit[2,2]
    end
    def set_maxsize(x, y)
      unless x.kind_of?(Fixnum) && y.kind_of?(Fixnum)
        fail TypeError, 'expect Fixnum values'
      end

      @size_limit[2] = x
      @size_limit[3] = y

      # minsize has higher priority than maxsize
      w = @container.winfo_width   # get and keep size before configure
      h = @container.winfo_height  # get and keep size before configure
      w = x if x >= @size_limit[0] && x < w
      h = y if y >= @size_limit[1] && y < h
      # When have to set one of sizes, have to reset both sizes 
      # to avoid troubles on `Configure' binding.
      @container.width  = w
      @container.height = h

      self
    end

    def get_minsize
      @size_limit[0,2]
    end
    def set_minsize(x, y)
      unless x.kind_of?(Fixnum) && y.kind_of?(Fixnum)
        fail TypeError, 'expect Fixnum values'
      end

      # @size_limit[0] = x
      @size_limit[0] = x
      @size_limit[1] = y

      # minsize has higher priority than maxsize
      w = @container.winfo_width   # get and keep size before configure
      h = @container.winfo_height  # get and keep size before configure
      w = x if x > w
      h = y if y > h
      # When have to set one of sizes, have to reset both sizes 
      # to avoid troubles on `Configure' binding.
      @container.width  = w
      @container.height = h

      self
    end

    def get_overrideredirect()
      (@base.borderwidth == 0)? '1': '0'
    end
    def set_overrideredirect(mode)
      TkORCA.security_check(5)

      if TkUtil.bool(mode)
        @base.borderwidth = 0
        @titlebar.unpack
      else
        @base.borderwidth = @border
        @titlebar.pack(:fill=>:x, :before=>@container)
      end
    end

    def protocol_list
      @WM_PROTOCOLS.keys
    end

    def get_protocol(type)
      @WM_PROTOCOLS[type][1]
    end
    def set_protocol(type, ip, cmd_str)
      TkORCA.security_check(5)

      if cmd_str =~ /\A\s*\Z/
        @WM_PROTOCOLS.delete(type)
      else
        @WM_PROTOCOLS[type] = [ip, cmd_str.dup.freeze]
      end
      self
    end
    def unset_protocol(type)
      set_protocol(type, nil, '')
    end

    def get_resizable
      @resizable.dup
    end
    def set_resizable(w, h)
      @resizable[0] = (w)? true: false
      @resizable[1] = (h)? true: false
      self
    end

    def get_position_from
      case @position_from
      when :user
        'user'
      when :program
        'program'
      when nil
        ''
      else
        # error : invalid value
        @position_from.to_s
      end
    end

    def position_from(src)
      case src
      when :user, 'user'
        @position_from = :user

      when :program, 'program'
        @position_from = :program

      when nil, ''
        @position_from = nil

      else
        fail ArgumentError, "invalid value `#{src}'"
      end
    end

    def get_size_from
      case @size_from
      when :user
        'user'
      when :program
        'program'
      when nil
        ''
      else
        # error : invalid value
        @position_from.to_s
      end
    end

    def size_from(src)
      case src
      when :user, 'user'
        @size_from = :user
        @base.pack_propagate(false)
        @saved_propagate = false

      when :program, 'program'
        @size_from = :program
        @container.width  = @container.winfo_width
        @container.height = @container.winfo_height
        @base.pack_propagate(true)
        @saved_propagate = true

      when nil, ''
        @size_from = nil
        @container.width  = @container.winfo_width
        @container.height = @container.winfo_height
        @base.pack_propagate(true)
        @saved_propagate = true

      else
        fail ArgumentError, "invalid value `#{src}'"
      end
      self
    end

    def container_stackorder
      rxmin, rymin, rxmax, rymax = @wall.scrollregion
      @wall.find_withtag(@tag).find_all{|f|
        if f.wm_state == 'normal'
          xmin, ymin, xmax, ymax = f.bbox
          ((rxmin < xmin && xmin < rxmax) || (rxmin < xmax && xmax < rxmax)) &&
            ((rymin < ymin && ymin < rymax) || (rymin < ymax && ymax < rymax))
        else
          false
        end
      }.collect{|f|
        begin
          w = @wall.itemcget(f, 'window')
          w = w.path if w.kind_of?(TkWindow)
          @@frames[w].winid
        rescue
          nil
        end
      }.compact
    end

    def raise(above = nil)
      if above.kind_of?(TkcWindow)
        super(above)
        w = above.cget('window')
        begin
          @base.raise(w) unless w == nil || w == ''
        rescue
          fail RuntimeError, "can't be raised above #{above}"
        end
      elsif above.kind_of?(TkcItem)
        super(above)
      elsif above = TkUtil::None
        super()
        @base.raise
      elsif above
        fail RuntimeError, "can't be raised above #{above}"
      else
        super()
        @base.raise
      end
      self
    end

    def lower(below = nil)
      if below.kind_of?(TkcWindow)
        super(below)
        w = below.cget('window')
        begin
          @base.lower(w) unless w == nil || w == ''
        rescue
          fail RuntimeError, "can't be lowered below #{below}"
        end
        begin
          @base.lower(below.cget('window'))
        rescue
          # ignore error
        end
      elsif below.kind_of?(TkcItem)
        super(below)
      elsif below = TkUtil::None
        super()
        @base.lower
      elsif below
        fail RuntimeError, "can't be lowered below #{below}"
      else
        super()
        @base.lower
      end
      self
    end

    def close
      TkORCA.security_check(5)

      ip, cmd = @WM_PROTOCOLS['WM_DELETE_WINDOW']
      if ip && cmd
        ip._eval(cmd)
      else
        @container.destroy
      end
      self
    end

    def iconify
      TkORCA.security_check(5)

      return self if @iconic

      @wm_state = 'iconic'
      self.state = :normal
      @saved_propagate = @base.pack_propagate
      @base.pack_propagate(true) # if @geo_def
      x1, y1, x2, y2 = self.bbox
      self.coords(x2, y1)
      self.anchor(:ne)
      @container.unpack
      @dummy_frame.pack(:expand=>false)
      @iconic = true
      @balloon.enable
      @minimize_btn.image = Images['deiconify_btn']

      self
    end

    def deiconify
      TkORCA.security_check(5)

      return self if @iconic == false

      current_propagate = @base.pack_propagate
      @base.pack_propagate(true)

      self.raise if @wm_state == 'withdrawn'  # have to raise it on top
      @wm_state = 'normal'
      self.state = :normal
      @dummy_frame.unpack
      @container.pack(:fill=>:both, :expand=>true)
      Tk.update_idletasks
      x1, y1, x2, y2 = self.bbox
      self.coords(x1, y1)
      self.anchor(:nw)
      @iconic = false
      @balloon.disable
      @minimize_btn.image = Images['minimize_btn']

      @base.pack_propagate(current_propagate)
      @saved_propagate = current_propagate

      self
    end

    def withdraw
      TkORCA.security_check(5)

      return self if @iconic == nil

      x1, y1, x2, y2 = self.bbox
      self.coords(x2, y1)
      self.anchor(:ne)
      @iconic = nil
      @balloon.disable

      @wm_state = 'withdrawn'
      self.state = :hidden
      @saved_propagate = @base.pack_propagate
      @base.pack_propagate(true) # if @geo_def
      @container.unpack
      @dummy_frame.pack(:expand=>false)

      self
    end

    def wm_state(new_st = nil)
      unless new_st
        @wm_state.dup
      else
        new_st = new_st.to_s
        case new_st
        when 'normal'
          self.deiconify
        when 'iconic'
          self.iconify
        when 'withdrawn'
          self.withdraw
        when 'zoomed'
          ###################
          # Not support yet.
        when 'icon'
          fail RuntimeError, "`icon' state cannot be set"
        else
          fail ArgumentError, "unkown state `#{new_st}'"
        end

        self
      end
    end
  end
end
