#
#  wrap_IO.rb : 
#
#                           by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
if $0 == __FILE__
  $LOAD_PATH << File.expand_path(File.join(File.dirname(__FILE__), '..'))
end

require 'tkorca/utils'

module TkORCA
  class IO_WRAPPER
    allows = ['__id__', '__send__', 'object_id', 'to_s', 'freeze']
    undef_method(*(instance_methods - allows - allows.map{|s| s.to_sym}))

    ########################################

    def initialize(io, &block)
      # the block has to return an array of arguments or nil (if unavailable).
      TkORCA.security_check(1)  # allow only when $SAFE==0 
      @io = io
      @block = block
      @wrapper = proc{|msg, *args| @io.__send__(msg, *args) }

      @kernel_to_s = Kernel.instance_method(:to_s)
      @kernel_kind_of = Kernel.instance_method(:kind_of?)

      @hash_to_a = Hash.instance_method(:to_a)
      @array_collect = Array.instance_method(:collect)
      @string_length = String.instance_method(:length)

      @inspector = proc{|obj, pp_mode|
        if obj == nil
          (pp_mode)? 'nil': ''
        elsif @kernel_kind_of.bind(obj).call(Hash)
          ary = proc{$SAFE=4; @hash_to_a.bind(obj).call}.call.taint
          ((pp_mode)? '{': '') << proc{
            $SAFE=4
            @array_collect.bind(ary).call{|k,v|
              "#{@inspector.call(k, pp_mode)}=>#{@inspector.call(v, pp_mode)}"
            }
          }.call.join((pp_mode)? ', ': '') << ((pp_mode)? '}': '')
        elsif @kernel_kind_of.bind(obj).call(Array)
          ((pp_mode)? '[': '') << proc{
            $SAFE=4
            @array_collect.bind(obj).call{|item| @inspector.call(item,pp_mode)}
          }.call.join((pp_mode)? ', ': '') << ((pp_mode)? ']': '')
        else
          begin
            begin
              dup_obj = obj.dup.taint unless obj.tainted?
            rescue TypeError
              dup_obj = obj
            end
            proc{
              $SAFE=4
              if pp_mode
                dup_obj.inspect
              else
                dup_obj.to_s
              end
          }.call
          rescue SecurityError
            @kernel_to_s.bind(obj).call
          end
        end
      }
    end

    ########################################

    def set_wrapper(block = Proc.new)
      # the block has to return an array of arguments or nil (if unavailable).
      TkORCA.security_check(1)  # allow only when $SAFE==0 
      @block = block
    end

    def method_missing(msg, *args)
      # call under caller's safe-level
      if @block
        if (new_args = @block.call(msg, *args))
          @io.__send__(*new_args)
        end
      else
        @io.__send__(msg, *args)
      end
    end

    def <<(obj)
      proc{
        $SAFE = 4 if $SAFE < 4
=begin
        obj = obj.to_s
        unless obj.kind_of?(String)
          raise TypeError, 'cannot convert to String' 
        end
=end
        @wrapper.call(@inspector.call(obj, false))
        @wrapper.call(:write, obj)
        self
      }.call
    end

    def putc(ch)
      proc{
        $SAFE = 4 if $SAFE < 4
=begin
        if ch.kind_of?(Integer) || ch.kind_of?(String)
          @wrapper.call(:putc, ch)
        else
          raise TypeError, 'cannot convert to Integer' 
        end
=end
        m = @kernel_kind_of.bind(ch)

        if m.call(String)
          str = ch[0,1]
        elsif m.call(Integer)
          str = ch.chr
        else
          fail TypeError, 'cannot convert to Integer' 
        end
        @wrapper.call(:putc, ch)
        ch
      }.call
    end

    def puts(*args)
      proc{
        $SAFE = 4 if $SAFE < 4
=begin
        args.each{|arg|
          if arg.nil?
            @wrapper.call(:write, "nil\n")
          else
            if arg.respond_to?(:to_ary) && (ary = arg.to_ary).kind_of?(Array)
              self.puts(*ary)
            else
              str = arg.to_s.chomp << "\n"
              unless str.kind_of?(String)
                raise TypeError, 'cannot convert to String' 
              end
              @wrapper.call(:write, str)
            end
          end
        }
=end
        args.each{|obj|
          if obj == nil
            @wrapper.call(:write, "nil\n")
          elsif @kernel_kind_of.bind(obj).call(String)
            obj = @inspector.call(obj, false)
            if obj =~ /\n\Z/
              @wrapper.call(:write, obj)
            else
              @wrapper.call(:write, @inspector.call(obj + "\n", false))
            end
          elsif @kernel_kind_of.bind(obj).call(Array)
            self.puts(*obj)
          elsif (obj.respond_to?(:to_ary) && 
                   @kernel_kind_of.bind(ary = arg.to_ary).call(Array))
            self.puts(*ary)
          else
            begin
              str = @inspector.call(obj.to_s, false)
            rescue
              str = @inspector.call(obj, false)
            end
            unless @kernel_kind_of.bind(str).call(String)
              fail TypeError, 'cannot convert to String' 
            end
            if str =~ /\n\Z/
              @wrapper.call(:write, @inspector.call(str, false))
            else
              @wrapper.call(:write, @inspector.call(str + "\n"))
            end
          end
        }
        nil
      }.call
    end

    def print(*args)
      proc{
=begin
        $SAFE = 4 if $SAFE < 4
        str = args.map{|arg|
          (arg.nil?)? 'nil': arg.to_s
        }.join($;.to_s) << $\.to_s
        @wrapper.call(:write, str)
=end
        if args.empty?
          self.print(@inspector.call($_, false))
        else
          str = args.collect{|obj|
            @inspector.call($,, false)
          }.join(($, == nil)? '': @inspector.call($,, false))
        end

        @wrapper.call(:write, str)
      }.call
    end

    def printf(format, *args)
      proc{
        $SAFE = 4 if $SAFE < 4
        format = @inspector.call(format, false)
        str = format % args
        #unless str.kind_of?(String)
        unless @kernel_kind_of.bind(str).call(String)
          raise TypeError, 'cannot get formatted String object' 
        end
        @wrapper.call(:write, str)
      }.call
    end

    def syswrite(str)
      proc{
        $SAFE = 4 if $SAFE < 4
        begin
          new_str = str.to_s
        rescue
          new_str = @inspector.call(str, false)
        end
        str = new_str
        #unless str.kind_of?(String)
        unless @kernel_kind_of.bind(str).call(String)
          raise TypeError, 'cannot convert to String' 
        end
        @wrapper.call(:syswrite, str)
      }.call
    end

    def write(str)
      proc{
        $SAFE = 4 if $SAFE < 4
        begin
          new_str = str.to_s
        rescue
          new_str = @inspector.call(str, false)
        end
        str = new_str
        #unless str.kind_of?(String)
        unless @kernel_kind_of.bind(str).call(String)
          raise TypeError, 'cannot convert to String' 
        end
        @wrapper.call(:write, str)
        @string_length.bind(str).call
      }.call
    end

    def write_nonblocking(str)
      proc{
        $SAFE = 4 if $SAFE < 4
        begin
          new_str = str.to_s
        rescue
          new_str = @inspector.call(str, false)
        end
        str = new_str
        #unless str.kind_of?(String)
        unless @kernel_kind_of.bind(str).call(String)
          raise TypeError, 'cannot convert to String' 
        end
        @wrapper.call(:write_nonblocking, str)
      }.call
    end

    ###############################

    def self.wrap_stdin(&blk)
      TkORCA.security_check(1)  # allow only when $SAFE==0 
      $stdin = TkORCA::IO_WRAPPER.new(STDIN, &blk)
    end
    def self.wrap_stdout(&blk)
      TkORCA.security_check(1)  # allow only when $SAFE==0 
      $stdout = TkORCA::IO_WRAPPER.new(STDOUT, &blk)
    end
    def self.wrap_stderr(&blk)
      TkORCA.security_check(1)  # allow only when $SAFE==0 
      $stderr = TkORCA::IO_WRAPPER.new(STDERR, &blk)
    end

    ###############################

    self.freeze
  end
end
