#
#  wrap_functions.rb : 
#
#                           by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
if $0 == __FILE__
  $LOAD_PATH << File.expand_path(File.join(File.dirname(__FILE__), '..'))
end

require 'tkorca/method_wrapper'
require 'tkorca/debug'

#######################################################################
#  requrie(lib)
#######################################################################
#----------------------------------------------------------------------
#  << definition-style 1 >>  ---  a block
#
#    The block accepts the original (not wrapped) method object 
#    and all arguments. 'orig_m.call(*args)' calls the original 
#    method on the safe-level when defined the wrapper.
#
#    NOTE: On ruby-1.8.x, this style cannot accept a block argument.
#          If the method requires or accepts a block, 
#          you have to use << definition-style 2>>.
#----------------------------------------------------------------------
TkORCA::MethodWrapper.wrap_function(:require){|orig_m, file|
  if TkORCA.is_mother?
    orig_m.call(file)
  else
    TkORCA::LOG.log(TkORCA::Daughter._name(MultiTkIp.__getip), 
                    "require `#{file}'")
    TkORCA::DEBUG.p [
      "wrapped `require' is called from daughter" , #"$SAFE==#{$SAFE})", 
      self, orig_m, file
    ]

    unless libpath = TkORCA::FileAccess.allow_to_require?(file)
      fail LoadError, "cannot require file -- `#{file}'"
    end

    orig_m.call(libpath)
  end
}
#
#----------------------------------------------------------------------
#  << definition-style 2 >>  ---  a method
#
#    The method accepts the original (not wrapped) method object 
#    and all arguments. 'orig_m.call(*args)' calls the original 
#    method on the safe-level when defined the wrapper.
#    The wrapper method is kept in TkORCA::MethodWrapper. It means
#    that it has no effect for the wrapper to override the method 
#    after defining the wrapper.
#
#    NOTE: This style can accept a block argument even ruby-1.8.x.
#          If you want to pass a block to the method, 
#          use a '&block' type argument to define wraper.
#            e.g.   def wrapper(orig_m, *args, &block)
#                     orig_m.call(*args, &block)
#                   end
#----------------------------------------------------------------------
#def wrapped_require(orig_m, file)
#  TkORCA::DEBUG.p(["wrapped_function_call $SAFE=#{$SAFE}", 
#                   self, orig_m, file])
#  orig_m.call(file)
#end
#TkORCA::MethodWrapper.wrap_function(:require, method(:wrapped_require))


#######################################################################
#  load(file, wrap=false)
#######################################################################
TkORCA::MethodWrapper.wrap_function(:load){|orig_m, *args|
  file, wrap_mode = args

  if TkORCA.is_mother?
    orig_m.call(file, wrap_mode)

  else  # is daughter
    TkORCA::LOG.log(TkORCA::Daughter._name(MultiTkIp.__getip), 
                    "load `#{file}'")

    TkORCA::DEBUG.p [
      "wrapped `load' is called from daughter ($SAFE==#{$SAFE})", 
      self, orig_m, file, wrap_mode
    ]

    # should check the file path here
    unless fullpath = TkORCA::FileAccess.allow_to_load?(file)
      fail LoadError, "no such file to load -- `#{file}'"
    end

    if (wrapper = MultiTkIp.__pseudo_toplevel) && !wrap_mode
      body = IO.read(fullpath)
      wrapper.taint if      safe = MultiTkIp.safe_level
      proc{
        $SAFE = MultiTkIp.safe_level
        wrapper.module_eval(body, fullpath)
      }.call
    else
      orig_m.call(fullpath, wrap_mode)
    end
  end
}


#######################################################################
#  p(*args)
#######################################################################
TkORCA::MethodWrapper.wrap_function(:p){|orig_m, *args|
  if TkORCA.is_mother? || TkORCA::DEBUG.debug_mode
    orig_m.call(*args)
  end
}


#######################################################################
#  exit(*args) , exit!(*args), abort(*args)
#######################################################################
exit_proc = proc{|orig_m, *args|
  if Thread.current == Thread.main
    orig_m.call(*args)
  #elsif TkORCA.is_mother?
  #  orig_m.call(*args)
  else
    #Tk.exit
    #MultiTkIp.delete
    MultiTkIp.exit
  end
}
TkORCA::MethodWrapper.wrap_function(:exit,  exit_proc)
TkORCA::MethodWrapper.wrap_function(:exit!, exit_proc)
TkORCA::MethodWrapper.wrap_function(:abort, exit_proc)


#######################################################################
#  trap 
#######################################################################
TkORCA::MethodWrapper.wrap_function(:trap){|orig_m, *args|
  if Thread.current == Thread.main
    orig_m.call(*args)
  #if TkORCA.is_mother?
  #  orig_m.call(*args)
  else
    # ignore
  end
}
