#
#  wrap_misc_tkcmd.rb : wrap misc Tk command
#
#                           by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
require 'tkorca/tclcmd_wrapper'

class << TkORCA::TclCmdWrapper.entry_wrapper_module(Module.new)
  def replace_in(ip)
    ########################################
    #   Based on "man 3 Tk_SafeInit"
    ########################################
    # Tk_SafeInit is identical to Tk_Init except that it removes
    # all  Tk  commands  that are considered unsafe.  Those com-
    # mands and the reasons for their exclusion are:

    # bell   Continuous ringing of the bell is a nuisance.
    TkORCA::TclCmdWrapper.wrap_tcl_command(ip, 'bell', &(_create_nop_proc()))

    # send   Send  can  be  used to cause unsafe interpreters to
    #        execute commands.
    TkORCA::TclCmdWrapper.wrap_tcl_command(ip, 'send', &(_create_nop_proc()))

    # grab   Grab  can  be used to block the user from using any
    #        other applications.
    #
    #  --> tkorca/wrap_grab_cmd.rb

    # menu   Menus can be used to cover the entire screen and to
    #        steal input from the user.
    #
    #  --> allow
    if MultiTkIp.hidden_cmds(ip).index('menu')
      MultiTkIp.expose_cmd(ip, 'menu')
      # MultiTkIp.expose_cmd(ip, 'tk_menuSetFocus')
    end

    # clipboard
    #        A malicious script could replace  the  contents  of
    #        the clipboard with the string "rm -r *" and lead to
    #        surprises when the contents of  the  clipboard  are
    #        pasted.
    #
    # selection
    #        See clipboard.
    #
    #TkORCA::TclCmdWrapper.wrap_tcl_command(ip, 'clipboard', 
    #                                       &(_create_nop_proc()))
    #TkORCA::TclCmdWrapper.wrap_tcl_command(ip, 'selection', 
    #                                       &(_create_nop_proc()))
    if MultiTkIp.hidden_cmds(ip).index('clipboard')
      MultiTkIp.expose_cmd(ip, 'clipboard')
    end
    if MultiTkIp.hidden_cmds(ip).index('selection')
      MultiTkIp.expose_cmd(ip, 'selection')
    end

    # tk     The tk command recreates the send command, which is
    #        unsafe.
    if MultiTkIp.hidden_cmds(ip).index('tk')
      MultiTkIp.expose_cmd(ip, 'tk')
    end
=begin
    TkORCA::TclCmdWrapper.wrap_tcl_command(ip, 'tk'){|ip, cmd, *args|
      subcmd = args.shift
      case subcmd
      when 'appname'
        '' # return an empty string

      when 'caret'
        # allow
        MultiTkIp.invoke_hidden(ip, 'tk', subcmd, *args)

      when 'scaling'
        # allow to refer only
        if '-displayof' =~ /^#{args[0]}/
            MultiTkIp.invoke_hidden(ip, 'tk', subcmd, '-displayof', args[1])
        else
            MultiTkIp.invoke_hidden(ip, 'tk', subcmd)
        end

      when 'inactive'
        # allow to refer only
        if '-displayof' =~ /^#{args[0]}/
            MultiTkIp.invoke_hidden(ip, 'tk', subcmd, '-displayof', args[1])
        else
            MultiTkIp.invoke_hidden(ip, 'tk', subcmd)
        end

      when 'useinputmethods'
        # allow to refer only
        if '-displayof' =~ /^#{args[0]}/
            MultiTkIp.invoke_hidden(ip, 'tk', subcmd, '-displayof', args[1])
        else
            MultiTkIp.invoke_hidden(ip, 'tk', subcmd)
        end

      when 'windowingsystem'
        Tk.windowingsystem

      else
        '' # return an empty string
      end
    }
=end

    # tkwait Tkwait can block the containing process forever
    #
    #  --> supprted by MultiTkIp

    # toplevel
    #        Toplevels  can  be  used to cover the entire screen
    #        and to steal input from the user.
    #
    #  --> tkorca/wrap_toplevel_cmd.rb

    # wm     If toplevels are ever allowed, wm can  be  used  to
    #        remove decorations, move windows around, etc.
    #
    #  --> tkorca/wrap_wm_cmd.rb
  end

  ######################################
  private
  ######################################
  def _create_nop_proc
    proc{|ip, cmd, *args|
      '' # return an empty string
    }
  end
end
