#
#  wrap_tearoffmenu_cmd.rb : wrap Tcl/Tk command `::tk::TearOffMenu'
#
#    If you want to use this feature, that is, to support tearoff entry 
#    on a menu, your Ruby/Tk must have `TclTkIp#:_make_menu_embeddable' 
#    method (e.g. ruby-1.8.5 or later).
#    When not, this is disabled. It means that Ruby/TkORCA can't control
#    tearoff (torn-off) menus.
#
#                           by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
require 'tkorca/tclcmd_wrapper'

if MultiTkIp.respond_to?(:_make_menu_embeddable)
################################################################
class << TkORCA::TclCmdWrapper.entry_wrapper_module(Module.new)
  def replace_in(ip) # ip is TkORCA::Daughter
    make_menu_embeddable_cmd = ip.eval{
      TkComm._get_eval_string(proc{|m_path| 
                                MultiTkIp.__getip._make_menu_embeddable(m_path)
                              })
    }.value

=begin
    target_command = '::tk::TearOffMenu'
    ip._eval(<<-EOS)
       set cmd #{target_command}
       auto_load $cmd
       set preexp  {^[[:space:]]*source[[:space:]]+}
       set postexp {[[:space:]]*$}
       if {[regexp "${preexp}(.*)${postexp}" $auto_index($cmd) full file]} {
          foreach key [array names auto_index] {
             if {[regexp "${preexp}$file${postexp}" $auto_index($key)]} {
                unset auto_index($key)
             }
          }
       }


       #####################
       # main wrapper
       #####################
       proc $cmd {w {x 0} {y 0}} {
           if {$x == 0} {
               set x [winfo rootx $w]
           }
           if {$y == 0} {
               set y [winfo rooty $w]
           }

           set parent [winfo parent $w]
           while {[string compare [winfo toplevel $parent] $parent] \
                   || [string compare [winfo class $parent] "Menu"] == 0} {
               set parent [winfo parent $parent]
           }
           if {[string compare $parent "."] == 0} {
               set parent ""
           }
           for {set i 1} 1 {incr i} {
               set top $parent.tearoff$i
               if {![winfo exists $top]} {
                   break
               }
           }
           toplevel $top

           set menu $top.menu

           $w clone $menu menubar

           #{make_menu_embeddable_cmd} $menu
           # $menu configure -type normal

           pack $menu -fill both -expand true

           set parent [winfo parent $w]
           if {[string compare [$menu cget -title] ""]} {
               wm title $top [$menu cget -title]
           } else {
               switch [winfo class $parent] {
                   Menubutton {
                       wm title $top [$parent cget -text]
                   }
                   Menu {
                       wm title $top [$parent entrycget active -label]
                   }
               }
           }

           ### $menu post $x $y  # <--- NOT SUPPORT
           wm geometry $top +${x}+${y}

           $w unpost

           if {[winfo exists $menu] == 0} {
               return ""
           }

           set tearoff_cmd [$w cget -tearoffcommand]
           if {[string compare $tearoff_cmd ""]} {
               uplevel #0 $tearoff_cmd [list $w $menu]
           }
           return $menu
       }
    EOS
=end
#=begin
    target_command = '::tk::TearOffMenu'
    ip._eval(<<-EOS)
       set cmd #{target_command}
       auto_load $cmd
       set preexp  {^[[:space:]]*source[[:space:]]+}
       set postexp {[[:space:]]*$}
       if {[regexp "${preexp}(.*)${postexp}" $auto_index($cmd) full file]} {
          foreach key [array names auto_index] {
             if {[regexp "${preexp}$file${postexp}" $auto_index($key)]} {
                unset auto_index($key)
             }
          }
       }

       #####################
       # sub functions
       #####################
       proc TkORCA_embedMenu_getRootOffset menu {
           # replace this later
           return {0 0}
       }

       proc ::tk::embedMenu_Unpost menu {
           global tcl_platform
           variable ::tk::Priv

           catch {focus $Priv(focus)}
           set Priv(focus) ""

           if {($Priv(tearoff) != 0) || [string compare $Priv(menuBar) ""]} {
               if {[string compare $menu ""]} {
                   set grab [grab current $menu]
                   if {[string compare $grab ""]} {
                       grab release $grab
                   }
               }
               RestoreOldGrab
               if {[string compare $Priv(menuBar) ""]} {
                   $Priv(menuBar) configure -cursor $Priv(cursor)
                   set Priv(menuBar) {}
               }
               if {[string compare [tk windowingsystem] "x11"]} {
                   set Priv(tearoff) 0
               }
           }
       }

       proc ::tk::embedMenu_Motion {menu x y state} {
           variable ::tk::Priv
           if {[string compare $menu $Priv(window)] == 0} {
               $menu activate @$x,$y
               tk::GenerateMenuSelect $menu
           }
           if {($state & 0x1f00) != 0} {
               $menu postcascade active
           }
       }

       proc ::tk::embedMenu_ButtonDown menu {
           variable ::tk::Priv
           global tcl_platform

           if {![winfo viewable $menu]} {
               return
           }
           $menu postcascade active

           if {[string compare $Priv(menuBar) {}] == 0} {
               set Priv(menuBar) $menu
               set Priv(cursor) [$menu cget -cursor]
               $menu configure -cursor arrow
           }

           if {[string compare $menu [grab current $menu]]} {
               SaveGrabInfo $menu
           }

           if {[string compare [tk windowingsystem] "x11"] == 0} {
               grab -global $menu
           }
       }

       proc ::tk::embedMenuCascade_NextMenu {menu direction} {
           variable ::tk::Priv
           # First handle traversals into and out of cascaded menus.
           if {[string compare $direction "right"] == 0} {
               if {[string compare [$menu type active] "cascade"] == 0} {
                   $menu postcascade active
                   set m2 [$menu entrycget active -menu]
                   if {[string compare $m2 ""]} {
                       MenuFirstEntry $m2
                   }
               }
           } else {
               set m2 [winfo parent $menu]
               if {[string compare [winfo class $m2] "Menu"] == 0} {
                   $menu activate none
                   GenerateMenuSelect $menu
                   tk_menuSetFocus $m2

                   $m2 postcascade none
               }
           }
       }

       proc ::tk::embedMenu_Invoke {w buttonRelease} {
           variable ::tk::Priv
           if {$buttonRelease && [string compare $Priv(window) {}] == 0} {
               $w postcascade none
               $w activate none
               event generate $w <<MenuSelect>>
               embedMenu_Unpost $w
               return
           }
           if {[string compare [$w type active] "cascade"] == 0} {
               $w postcascade active
               set menu [$w entrycget active -menu]
               bind $menu <Escape> {
                   ::tk::embedMenuCascade_NextMenu %W left
                   break
               }
               bind $menu <Left>   {
                   ::tk::embedMenuCascade_NextMenu %W left
                   break
               }
               bind $menu <Right>  {
                   ::tk::embedMenuCascade_NextMenu %W right
                   break
               }
               MenuFirstEntry $menu
           } elseif {[string compare [$w type active] "tearoff"] == 0} {
               ::tk::TearOffMenu $w
               embedMenu_Unpost $w
           } else {
               embedMenu_Unpost $w
               uplevel #0 [list $w invoke active]
           }
       }

       proc ::tk::embedMenu_TraverseWithin {w char} {
           if {[string compare $char ""] == 0} {
               return
           }
           set char [string tolower $char]
           set last [$w index last]
           if {[string compare $last "none"] == 0} {
               return
           }
           for {set i 0} {$i <= $last} {incr i} {
               if {[catch {set char2 [string index \
                    [$w entrycget $i -label] [$w entrycget $i -underline]]}]} {
                   continue
               }
               if {[string compare $char [string tolower $char2]] == 0} {
                   if {[string compare [$w type $i] "cascade"] == 0} {
                       $w activate $i
                       $w postcascade active
                        event generate $w <<MenuSelect>>
                        set m2 [$w entrycget $i -menu]
                        if {[string compare $m2 ""]} {
                            bind $m2 <Escape> {
                                ::tk::embedMenuCascade_NextMenu %W left
                                break
                            }
                            bind $m2 <Left> {
                                ::tk::embedMenuCascade_NextMenu %W left
                                break
                            }
                            bind $m2 <Right> {
                                ::tk::embedMenuCascade_NextMenu %W right
                                break
                            }
                            MenuFirstEntry $m2
                        }
                   } else {
                       embedMenu_Unpost $w
                       uplevel #0 [list $w invoke $i]
                   }
                   return
               }
           }
       }

       proc ::tk::embedMenu_NextMenu {menu direction} {
           variable ::tk::Priv

           # First handle traversals into and out of cascaded menus.
           if {[string compare $direction "right"] == 0} {
               set count 1
               set parent [winfo parent $menu]
               set class [winfo class $parent]
               if {[string compare [$menu type active] "cascade"] == 0} {
                   $menu postcascade active
                   set m2 [$menu entrycget active -menu]
                   if {[string compare $m2 ""]} {
                       bind $m2 <Escape> {
                           ::tk::embedMenuCascade_NextMenu %W left
                           break
                       }
                       bind $m2 <Left>   {
                           ::tk::embedMenuCascade_NextMenu %W left
                           break
                       }
                       bind $m2 <Right> {
                           ::tk::embedMenuCascade_NextMenu %W right
                           break
                       }
                       MenuFirstEntry $m2
                   }
                   return
               } else {
                   set parent [winfo parent $menu]
                   while {[string compare $parent "."]} {
                       if {[string compare [winfo class $parent] "Menu"] == 0 \
                            && [string compare [$parent cget -type] \
                                               "menubar"] == 0} {
                           tk_menuSetFocus $parent
                           MenuNextEntry $parent 1
                           return
                       }
                       set parent [winfo parent $parent]
                   }
               }
           } else {
               set count -1
               set m2 [winfo parent $menu]
               if {[string compare [winfo class $m2] "Menu"] == 0} {
                   $menu activate none
                   GenerateMenuSelect $menu
                   tk_menuSetFocus $m2

                   $m2 postcascade none

                   if {[string compare [$m2 cget -type] "menubar"]} {
                       return
                   }
               }
           }

           # Can't traverse into or out of a cascaded menu.  Go to the next
           # or previous menubutton, if that makes sense.

           set m2 [winfo parent $menu]
           if {[string compare [winfo class $m2] "Menu"] == 0} {
               if {[string compare [$m2 cget -type] "menubar"] == 0} {
                   tk_menuSetFocus $m2
                   MenuNextEntry $m2 -1
                   return
               }
           }

           set w $Priv(postedMb)
           if {[string compare $w ""] == 0} {
               return
           }

           set buttons [winfo children [winfo parent $w]]
           set length [llength $buttons]
           set i [expr {[lsearch -exact $buttons $w] + $count}]
           while {1} {
               while {$i < 0} {
                   incr i $length
               }
               while {$i >= $length} {
                   incr i -$length
               }
               set mb [lindex $buttons $i]
               if {[string compare [winfo class $mb] "Menubutton"] == 0 \
                    && [string compare [$mb cget -state] "disabled"] \
                    && [string compare [$mb cget -menu] ""] \
                    && [string compare [[$mb cget -menu] index last] "none"]} {
                   break
               }
               if {[string compare $mb $w] == 0} {
                   return
               }
               incr i $count
           }
           MbPost $mb
           MenuFirstEntry [$mb cget -menu]
       }

       proc ::tk::embedMenu_NextEntry {menu count} {
           if {[string compare [$menu index last] "none"] == 0} {
               return
           }
           set length [expr {[$menu index last]+1}]
           set quitAfter $length
           set active [$menu index active]
           if {[string compare $active "none"] == 0} {
               set i 0
           } else {
               set i [expr {$active + $count}]
           }
           while {1} {
               if {$quitAfter <= 0} {
                   # We've tried every entry in the menu.  Either there are
                   # none, or they're all disabled.  Just give up.
                   return
               }
               while {$i < 0} {
                   incr i $length
               }
               while {$i >= $length} {
                   incr i -$length
               }
               if {[catch {$menu entrycget $i -state} state] == 0} {
                   if {[string compare $state "disabled"] && \
                           ($i!=0 \
                           || [string compare [$menu cget -type] "tearoff"] \
                           || [string compare [$menu type 0] "tearoff"])} {
                       break
                   }
               }
               if {$i == $active} {
                   return
               }
               incr i $count
               incr quitAfter -1
           }
           $menu activate $i
           GenerateMenuSelect $menu
       }

       #####################
       # main wrapper
       #####################
       proc $cmd {w {x 0} {y 0}} {
           set title {}
           set parent [winfo parent $w]
           if {[string compare [$w cget -title] ""]} {
               set title [$w cget -title]
           } else {
               switch -regexp [winfo class $parent] {
                   .*[Mm]enu[Bb]utton {
                       set title [$parent cget -text]
                   }
                   .*[Mm]enu {
                       set title [$parent entrycget active -label]
                   }
                   .* {
                       if {[catch {$parent cget -text} ret] == 0} {
                           if {[string compare $ret ""]} { set title $ret }
                       }
                   }
               }
           }

           set offset [TkORCA_embedMenu_getRootOffset $w]
           if {$x == 0} {
               # set x [winfo rootx $w]
               set x [expr [winfo rootx $w] - [lindex $offset 0]]
           }
           if {$y == 0} {
               # set y [winfo rooty $w]
               set y [expr [winfo rooty $w] - [lindex $offset 1]]
           }

           set slave $w
           set parent [winfo parent $slave]

           while {[string compare [winfo toplevel $parent] $parent] \
                   || [string compare [winfo class $parent] "Menu"] == 0 \
                   || ([string compare $parent.menu $slave] == 0 \
                       && [string compare [winfo class $slave] "Menu"] == 0 \
                       && ! [catch {pack info $slave}])
                 } {
               set slave $parent
               set parent [winfo parent $parent]
           }
           if {[string compare $parent "."] == 0} {
               set parent ""
           }
           for {set i 1} 1 {incr i} {
               set top $parent.tearoff$i
               if {![winfo exists $top]} {
                   break
               }
           }
           toplevel $top

           set menu $top.menu

           $w clone $menu menubar

           #{make_menu_embeddable_cmd} $menu
           # $menu configure -type normal
           # $menu configure -type tearoff

           pack $menu -fill both -expand true

           if {[string compare [$menu cget -title] ""]} {
               set title [$menu cget -title]
           }
           if {[string compare $title ""]} {
               wm title $top $title
           }

           ### $menu post $x $y  # <--- NOT SUPPORT
           wm geometry $top +${x}+${y}

           $w unpost

           if {[winfo exists $menu] == 0} {
               return ""
           }

           ###############################

           #bind $menu <FocusIn> { break }
           bind $menu <FocusIn> { }

           #bind $menu <FocusOut> { break }
           bind $menu <FocusOut> { }

           bind $menu <Enter> {
               # set tk::Priv(focus) %W
               set tk::Priv(window) %W
               if {[string compare "%m" "NotifyUngrab"]} {
                   if {[string compare [tk windowingsystem] "x11"] == 0} {
                       tk_menuSetFocus %W
                   }
               }
               tk::embedMenu_Motion %W %x %y %s
               break
           }

           bind $menu <Leave> {
               variable ::tk::Priv
               # set tk::Priv(focus) {}
               set tk::Priv(window) {}
               if {[string compare [%W index active] "none"] == 0} {
                   return
               }
               if {[string compare [%W type active] "cascade"] == 0 \
                     && [string compare [winfo containing %X %Y] \
                                        [%W entrycget active -menu]] == 0} {
                   return
               }
               %W activate none
               tk::GenerateMenuSelect %W
           }

           bind $menu <Motion> {
               #set tk::Priv(focus) %W
               tk::embedMenu_Motion %W %x %y %s
               break
           }

           bind $menu <ButtonPress> {
               tk::embedMenu_ButtonDown %W
               break
           }
           bind $menu <ButtonRelease> {
               tk::embedMenu_Invoke %W 1
               break
           }
           bind $menu <space> {
               tk::embedMenu_Invoke %W 0
               break
           }
           bind $menu <Return> {
               tk::embedMenu_Invoke %W 0
               break
           }
           bind $menu <Escape> {
               set parent [winfo parent %W]
               if {[string compare [winfo class $parent] "Menu"]} {
                   tk::embedMenu_Unpost %W
               } else {
                   %W configure -type tearoff
                   tk::embedMenu_NextMenu %W left
                   %W configure -type menubar
               }
               break
           }
           bind $menu <Left> {
               tk::embedMenu_NextMenu %W left
               break
           }
           bind $menu <Right> {
               tk::embedMenu_NextMenu %W right
               break
           }
           bind $menu <Up> {
               %W configure -type tearoff
               tk::MenuNextEntry %W -1
               %W configure -type menubar
               break
           }
           bind $menu <Down> {
               %W configure -type tearoff
               tk::MenuNextEntry %W 1
               %W configure -type menubar
               break
           }
           bind $menu <KeyPress> {
               %W configure -type tearoff
               tk::embedMenu_TraverseWithin %W %A
               %W configure -type menubar
               break
           }

           set tearoff_cmd [$w cget -tearoffcommand]
           if {[string compare $tearoff_cmd ""]} {
               uplevel #0 $tearoff_cmd [list $w $menu]
           }
           return $menu
       }
    EOS
#=end

    TkORCA::TclCmdWrapper.wrap_tcl_command_transaction(ip, 'TkORCA_embedMenu_getRootOffset', <<-EOS)
      set menu [lindex $args 0]
      set scrn [interp invokehidden $slave winfo screen $menu]
      set tbl  [ruby "TkORCA::RFB_Canvas.get_wallpath_table_id"]
      set wall [set ${tbl}($scrn)]
      set top  [winfo toplevel $wall]
      return "[winfo rootx $top] [winfo rooty $top]"
      #return "0 0"
    EOS
  end
end
################################################################
end
