#
#  wrap_winfo_cmd.rb : wrap Tcl/Tk command `winfo'
#
#                           by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
require 'tkorca/tclcmd_wrapper'

class << TkORCA::TclCmdWrapper.entry_wrapper_module(Module.new)
  def replace_in(ip)
    get_wall = <<-EOS
      set scrn [interp invokehidden $slave winfo screen $params]
      set tbl  [ruby "TkORCA::RFB_Canvas.get_wallpath_table_id"]
      set wall [set ${tbl}($scrn)]
    EOS

    TkORCA::TclCmdWrapper.wrap_tcl_command_transaction(ip, 'winfo', <<-EOS)
    set subcmd [lindex $args 0]
    set params [lrange $args 1 end]

    switch -exact -- $subcmd {
      geometry {
        if {[string compare $params [interp eval $slave \
                                     winfo toplevel $params]] == 0} {
          # is toplevel
          if {[catch {interp eval $slave $params cget -use} use]} {
            # no '-use' option (maybe menu widget and so on)
            eval interp invokehidden $slave winfo geometry $params
          } else {
            if {[string compare $use ""] == 0} {
              # no frame
              eval interp invokehidden $slave winfo geometry $params

            } else {
              #{get_wall}
              set container [winfo pathname -displayof $wall $use]
              if [winfo ismapped $container] {
                set w [winfo width  $container]
                set h [winfo height $container]
              } else {
                set w [winfo reqwidth  $container]
                set h [winfo reqheight $container]
              }

              format "%dx%d+%d+%d" \
                     $w $h [winfo rootx $container] [winfo rooty $container]
            }
          }

        } else {
          # not toplevel
          eval interp invokehidden $slave winfo geometry $params
        }
      }

      ismapped {
        set top [interp eval $slave winfo toplevel $params]
        if {[catch {interp eval $slave $top cget -use} use]} {
          # no '-use' option (maybe menu widget and so on)
          eval interp invokehidden $slave winfo ismapped $params
        } else {
          if {[string compare $use ""] == 0} {
            # no frame
            eval interp invokehidden $slave winfo ismapped $params
          } else {
            #{get_wall}
            winfo ismapped [winfo pathname -displayof $wall $use]
          }
        }
      }

      vrootheight {
        #{get_wall}
        set region [$wall cget -scrollregion]
        expr int([lindex $region 3] - [lindex $region 1] + 1)
      }

      vrootwidth {
        #{get_wall}
        set region [$wall cget -scrollregion]
        expr int([lindex $region 2] - [lindex $region 0] + 1)
      }

      vrootx {
        #{get_wall}
        expr int([$wall canvasx 0] - [winfo rootx $wall])
      }

      vrooty {
        #{get_wall}
        expr int([$wall canvasy 0] - [winfo rooty $wall])
      }

      default {
        eval interp invokehidden $slave winfo $subcmd $params
      }
    }
    EOS
  end
end
