#
#  wrap_wm_cmd.rb : wrap Tcl/Tk command `wm'
#
#                           by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
require 'tkorca/tclcmd_wrapper'

class << TkORCA::TclCmdWrapper.entry_wrapper_module(Module.new)
  def replace_in(ip)
    TkORCA::TclCmdWrapper.wrap_tcl_command(ip, 'wm', &(_create_proc()))
  end

  ######################################
  private
  ######################################
  def _create_proc
    proc{|ip, cmd, *args|
      subcmd = args.shift

      begin
        __send__(subcmd, ip, *args)
      rescue NoMethodError => e
        raise e if private_methods.index(subcmd)
        if ip.safe?
          raise SecurityError, 
                "refuse `#{subcmd}' subcommand for safe-interpreter"
        end
        MultiTkIp.invoke_hidden(ip, 'wm', subcmd, *args)
      end
    }
  end

  def _check_toplevel(ip, win)
    if ip.eval{Tk.tk_call('winfo', 'toplevel', win)} != win
      raise RuntimeError, "window #{win} isn't a top-level window"
    end
    win
  end

  ######################################

  def close_button_notify(ip, win, msg)
    fwin = TkORCA::TclCmdWrapper.get_window_frame(ip, _check_toplevel(ip, win))
    if fwin
      fwin.close_button_notify(msg)
    end
    ''  # return an empty string
  end

  ######################################

  def aspect(ip, win, *args)
    top = TkORCA::TclCmdWrapper.get_window_frame(ip,_check_toplevel(ip,win))
    unless top
      return MultiTkIp.invoke_hidden(ip, 'wm', 'aspect', win, *args)
    end

    case args.size
    when 0
      if (ary = top.get_aspect).max > 0
        ary
      else
        ''  # return an empty string
      end
    when 4
      top.set_aspect(*(args.collect{|arg| Integer(arg)}))
      ''  # return an empty string
    else
      raise ArguemtError, "wrong number of arguments"
    end
  end

  def attributes(ip, win, *args)
    ####################
    # Not support yet.
    # It allows you to refer current attributes, but disallows to set.
    case args.size
    when 0
      MultiTkIp.invoke_hidden(ip, 'wm', 'attributes', win)
    when 1
      MultiTkIp.invoke_hidden(ip, 'wm', 'attributes', win, args[0])
    else
      ''
      # MultiTkIp.invoke_hidden(ip, 'wm', 'attributes', win, *args)
    end
  end

  def client(ip, win, name = nil)
    if name
      ### Is it OK? Maybe required some checks for security
      MultiTkIp.invoke_hidden(ip, 'wm', 'client', name)
    else
      MultiTkIp.invoke_hidden(ip, 'wm', 'client', win)
    end
  end

  def colormapwindows(ip, win, *args)
    ### Is it OK? Maybe required some checks for security
    MultiTkIp.invoke_hidden(ip, 'wm', 'colormapwindows', *args)
  end

  def command(ip, win, *args)
    ### Is it OK? Maybe required some checks for security
    MultiTkIp.invoke_hidden(ip, 'wm', 'command', *args)
  end

  def deiconify(ip, win)
    top = TkORCA::TclCmdWrapper.get_window_frame(ip,_check_toplevel(ip,win))
    if top
      top.deiconify
    else # maybe menu and so on
      MultiTkIp.invoke_hidden(ip, 'wm', 'deiconify', win)
    end
    ''  # return an empty string

  end

  def focusmodel(ip, win, mode = nil)
    if mode
      ### disallows you to set the mode for security.
      ''
    else
      MultiTkIp.invoke_hidden(ip, 'wm', 'focusmodel', win)
    end
  end

  def frame(ip, win)
    top = TkORCA::TclCmdWrapper.get_window_frame(ip,_check_toplevel(ip,win))
    if top
      top.frame || ip._invoke('winfo', 'id', win)
      #top.frame || ip._invoke('winfo', 'id', (top.path rescue top.to_s))
    else # maybe menu and so on
      MultiTkIp.invoke_hidden(ip, 'wm', 'frame', win)
    end
  end

  def geometry(ip, win, *args)
    top = TkORCA::TclCmdWrapper.get_window_frame(ip,_check_toplevel(ip,win))
    unless top
      return MultiTkIp.invoke_hidden(ip, 'wm', 'geometry', win, *args)
    end

    case args.size
    when 0
      top.geometry
    when 1
      #return MultiTkIp.invoke_hidden(ip, 'wm', 'geometry', win, *args)
      top.geometry(args[0])
      ''  # return an empty string
    else
      raise ArguemtError, "wrong number of arguments"
    end
  end

  def grid(ip, win, *args)
    ####################
    # Not support yet.
  end

  def group(ip, win, pathname = nil)
    ####################
    # Not support yet.
    # It allows you to refer current value, but disallows to set.
    if pathname
      ''
      # MultiTkIp.invoke_hidden(ip, 'wm', 'group', win, pathname)
    else
      MultiTkIp.invoke_hidden(ip, 'wm', 'group', win)
    end
  end

  def iconify(ip, win)
    top = TkORCA::TclCmdWrapper.get_window_frame(ip,_check_toplevel(ip,win))
    if top
      top.iconify
    else # maybe menu and so on
      MultiTkIp.invoke_hidden(ip, 'wm', 'iconify', win)
    end
    ''  # return an empty string
  end

  def iconbitmap(ip, win, *args)
    ####################
    # Not support yet.
  end

  def iconmask(ip, win, *args)
    ####################
    # Not support yet.
  end

  def iconname(ip, win, *args)
    ####################
    # Not support yet.
  end

  def iconphoto(ip, win, *args)
    ####################
    # Not support yet.
  end

  def iconposition(ip, win, *args)
    ####################
    # Not support yet.
  end

  def iconwindow(ip, win, *args)
    ####################
    # Not support yet.
  end

  def manage(ip, win, *args)
    ####################
    # Not support yet.
  end

  def forget(ip, win, *args)
    ####################
    # Not support yet.
  end

  def maxsize(ip, win, *args)
    top = TkORCA::TclCmdWrapper.get_window_frame(ip,_check_toplevel(ip,win))
    unless top
      # maybe menu and so on
      return MultiTkIp.invoke_hidden(ip, 'wm', 'maxsize', win, *args)
    end
    case args.size
    when 0
      top.get_maxsize
    when 2
      top.set_maxsize(Integer(args[0]), Integer(args[1]))
      ''  # return an empty string
    else
      raise ArguemtError, "wrong number of arguments"
    end
  end

  def minsize(ip, win, *args)
    top = TkORCA::TclCmdWrapper.get_window_frame(ip,_check_toplevel(ip,win))
    unless top
      # maybe menu and so on
      return MultiTkIp.invoke_hidden(ip, 'wm', 'minsize', win, *args)
    end
    case args.size
    when 0
      top.get_minsize
    when 2
      top.set_minsize(Integer(args[0]), Integer(args[1]))
      ''  # return an empty string
    else
      raise ArguemtError, "wrong number of arguments"
    end
  end

  def overrideredirect(ip, win, *args)
    fwin = TkORCA::TclCmdWrapper.get_window_frame(ip, _check_toplevel(ip, win))
    unless fwin
      # maybe menu and so on
      return MultiTkIp.invoke_hidden(ip, 'wm', 'overrideredirect', win, *args)
    end
    case args.size
    when 0
      fwin.get_overrideredirect
    when 1
      fwin.set_overrideredirect(args[0])
      ''  # return an empty string
    else
      raise ArguemtError, "wrong number of arguments"
    end
  end

  def positionfrom(ip, win, *args)
    fwin = TkORCA::TclCmdWrapper.get_window_frame(ip, _check_toplevel(ip, win))
    unless fwin
      # maybe menu and so on
      return MultiTkIp.invoke_hidden(ip, 'wm', 'positionfrom', win, *args)
    end
    case args.size
    when 0
      fwin.get_position_from
    when 1
      fwin.position_from(args[0])
      ''  # return an empty string
    else
      raise ArguemtError, "wrong number of arguments"
    end
  end

  def protocol(ip, win, *args)
    fwin = TkORCA::TclCmdWrapper.get_window_frame(ip, _check_toplevel(ip, win))
    unless fwin
      # maybe menu and so on
      return MultiTkIp.invoke_hidden(ip, 'wm', 'protocol', win, *args)
    end
    case args.size
    when 0
      fwin.protocol_list
    when 1
      fwin.get_protocol(args[0])
    when 2
      fwin.set_protocol(args[0], ip, args[1])
      ''  # return an empty string
    else
      raise ArguemtError, "wrong number of arguments"
    end
  end

  def resizable(ip, win, *args)
    fwin = TkORCA::TclCmdWrapper.get_window_frame(ip, _check_toplevel(ip, win))
    unless fwin
      # maybe menu and so on
      return MultiTkIp.invoke_hidden(ip, 'wm', 'resizable', win, *args)
    end
    case args.size
    when 0
      fwin.get_resizable
    when 2
      fwin.set_resizable(TkUtil.bool(args[0]), TkUtil.bool(args[1]))
      ''  # return an empty string
    else
      raise ArguemtError, "wrong number of arguments"
    end
  end

  def sizefrom(ip, win, *args)
    fwin = TkORCA::TclCmdWrapper.get_window_frame(ip, _check_toplevel(ip, win))
    unless fwin
      # maybe menu and so on
      return MultiTkIp.invoke_hidden(ip, 'wm', 'sizefrom', win, *args)
    end
    case args.size
    when 0
      fwin.get_size_from
    when 1
      fwin.size_from(args[0])
      ''  # return an empty string
    else
      raise ArguemtError, "wrong number of arguments"
    end
  end

  def _get_toplevel_id_table(ip, win)
    tbl = {}
    target = [win]
    until target.empty?
      list = target
      target = []
      list.each{|path|
        top = ip._invoke('winfo', 'toplevel', path)
        unless tbl.has_key?(top)
          tbl[top] = (ip._invoke(top, 'cget', '-use') rescue false)
        end
        target.concat((TclTkLib._split_tklist(ip._invoke('winfo', 'children', path)) rescue []))
      }
    end
    tbl
  end
  private :_get_toplevel_id_table

  def stackorder(ip, win, *args)
    unless TkUtil.bool(ip._invoke('winfo', 'exists', win))
      fail RuntimeError, "bad window path `#{win}'"
    end
    unless TkUtil.bool(ip._invoke('winfo', 'ismapped', win))
      fail RuntimeError, "`#{win}' is not mapped"
    end
    unless win == ip._invoke('winfo', 'toplevel', win)
      fail RuntimeError, "`#{win}' is not a toplevel"
    end
    without_frame_order = TclTkLib._split_tklist(MultiTkIp.invoke_hidden(ip, 'wm', 'stackorder', win))
    with_frame_order = nil

    id_table = _get_toplevel_id_table(ip, win).invert

    if (fwin = TkORCA::TclCmdWrapper.get_window_frame(ip, _check_toplevel(ip, win)))
      with_frame_order = fwin.container_stackorder.collect{|win_id|
        id_table[win_id]
      }
      with_frame_order.compact!
    end

    if args.empty?
      order = (with_frame_order || []).concat(without_frame_order)
      TclTkLib._merge_tklist(*order)

    else
      mode, win2 = args

      unless TkUtil.bool(ip._invoke('winfo', 'exists', win2))
        fail RuntimeError, "bad window path `#{win2}'"
      end
      unless TkUtil.bool(ip._invoke('winfo', 'ismapped', win2))
        fail RuntimeError, "`#{win2}' is not mapped"
      end
      unless win2 == ip._invoke('winfo', 'toplevel', win2)
        fail RuntimeError, "`#{win2}' is not a toplevel"
      end

      unless with_frame_order
        if (fwin = TkORCA::TclCmdWrapper.get_window_frame(ip, _check_toplevel(ip, win2)))
          with_frame_order = fwin.container_stackorder.collect{|win_id|
            id_table[win_id]
          }
        end
      end

      order = (with_frame_order || []).concat(without_frame_order)

      idx1 = order.index(win)  || order.length
      idx2 = order.index(win2) || order.length

      if mode == 'isabove'
        idx1 > idx2
      elsif mode == 'isbelow'
        idx1 < idx2
      else
        fail RuntimeError, "bad option `#{mode}'"
      end
    end
  end

  def state(ip, win, *args)
    fwin = TkORCA::TclCmdWrapper.get_window_frame(ip, _check_toplevel(ip, win))
    unless fwin
      # maybe menu and so on
      return MultiTkIp.invoke_hidden(ip, 'wm', 'state', win, *args)
    end
    case args.size
    when 0
      fwin.wm_state()
    when 1
      fwin.wm_state(*args)
      ''  # return an empty string
    else
      raise ArgumentError, 'wrong number of subcommand arguments'
    end
  end

  def title(ip, win, *args)
    fwin = TkORCA::TclCmdWrapper.get_window_frame(ip, _check_toplevel(ip, win))
    unless fwin
      # maybe menu and so on
      return MultiTkIp.invoke_hidden(ip, 'wm', 'title', win, *args)
    end
    case args.size
    when 0
      fwin.title
    when 1
      fwin.title = args[0]
      ''  # return an empty string
    else
      raise ArgumentError, 'wrong number of subcommand arguments'
    end
  end

  def transient(ip, win, *args)
    MultiTkIp.invoke_hidden(ip, 'wm', 'transient', win, *args)
  end

  def withdraw(ip, win)
    top = TkORCA::TclCmdWrapper.get_window_frame(ip,_check_toplevel(ip,win))
    if top
      top.withdraw
    else # maybe menu and so on
      MultiTkIp.invoke_hidden(ip, 'wm', 'withdraw', win)
    end
    ''  # return an empty string
  end
end
