#!/usr/local/bin/ruby
#
#  Ruby/TkORCA server daemon script
#
#                             by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
########################################################
# add SERVER_DIR to load-path
########################################################
SERVER_DIR = File.dirname(File.expand_path(__FILE__))
$LOAD_PATH.unshift(SERVER_DIR)


########################################################
# setup base definitions
########################################################
load(File.join(SERVER_DIR, 'server_base_defs'))

unless Object.const_defined?(:NULL_DEVICE)
  NULL_DEVICE = '/dev/null'
end


########################################################
#  keep original commandline options
########################################################
ORIGINAL_ARGV = ARGV.dup


########################################################
#  get command-line options
########################################################
require 'server/opt_parse.rb'

#  null device
if OPTION['null_device']
  null_device = OPTION['ruby_bin']
else
  null_device = NULL_DEVICE
end


########################################################
#  decide ruby binary path
########################################################
if OPTION['ruby_bin']
  ruby_bin = File.expand_path(OPTION['ruby_bin'])
else
  if Object.const_defined?(:RUBY_BINARY)
    ruby_bin = File.expand_path(RUBY_BINARY)
  else
    require 'rbconfig'
    bin_name = Config::CONFIG['rubyw_install_name']
    if !bin_name || bin_name.empty?
      bin_name = Config::CONFIG['ruby_install_name']
    end

    if FileTest.exist?(path = File.join(Config::CONFIG['bindir'], bin_name))
      ruby_bin = path
    elsif FileTest.exist?(path = File.join('/usr/local/bin', bin_name))
      ruby_bin = path
    elsif FileTest.exist?(path = File.join('/usr/bin', bin_name))
      ruby_bin = path
    else # entrust command-search-path
      ruby_bin = `which #{bin_name} 2>#{null_device}`.chomp
      if $?.exitstatus != 0
        # Pray to the gods ...
        ruby_bin = nil
      end
    end
  end
end


########################################################
#  environment check
########################################################
unless OPTION['hangup_signal'] || Object.const_defined?(:TkORCA_SIGHUP)
  list = Signal.list
  if list.has_key?('HUP')
    TkORCA_SIGHUP = :HUP
  elsif list.has_key?('BREAK')
    TkORCA_SIGHUP = :BREAK
  else
    raise RuntimeError, "Fail to select proper singal for TkORCA_SIGHUP." + 
      " It is required for interactive control of application processes." + 
      " Please define TkORCA_SIGHUP on `server_base_defs' file" + 
      " or use `-S' (or `--hangup_signal') option."
  end
end


########################################################
#  make eval arguments
########################################################
server = File.join(SERVER_DIR, 'tkorca_server.rb')

if OPTION['ruby_bin']
  cmd = [ruby_bin, server, *ORIGINAL_ARGV]
elsif ruby_bin
  cmd = [ruby_bin, server, '-b', ruby_bin, *ORIGINAL_ARGV]
else
  cmd = [[bin_name, bin_name], server, *ORIGINAL_ARGV]
end


########################################################
#  start daemon
########################################################
Kernel.exit!(0) if fork{
  unless OPTION['debug_mode']
    $stdin.reopen(null_device)
    $stdout.reopen(null_device)
    # $stderr.reopen(null_device) # keep for writing error messages
  end
  Process.setsid
  exec(*cmd)
}
