#!/usr/local/bin/ruby
#
#  tkorca_logrotate.rb [ ... options ... ]
#
#      options are same as options of tkorca_server.rb.
#
#                             by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
require 'tmpdir'

LOG_BACKUP_MAX = 9

##############################################
#  load version information
##############################################
load File.join(File.dirname(__FILE__), 'version.rb')


########################################################
# add SERVER_DIR to load-path
########################################################
SERVER_DIR = File.dirname(File.expand_path(__FILE__))
$LOAD_PATH.unshift(SERVER_DIR)


########################################################
# setup base definitions
########################################################
load(File.join(SERVER_DIR, 'server_base_defs'))


########################################################
# define defaults
########################################################
unless Object.const_defined?(:TkORCA_SIGHUP)
  list = Signal.list
  if list.has_key?('HUP')
    TkORCA_SIGHUP = :HUP
  elsif list.has_key?('BREAK')
    TkORCA_SIGHUP = :BREAK
  else
    TkORCA_SIGHUP = nil
  end
end

unless Object.const_defined?(:SERVER_CONFDIR)
  # SERVER_CONFDIR = SERVER_DIR
  SERVER_CONFDIR = File.join(SERVER_DIR, 'sysconf')
end

unless Object.const_defined?(:SERVER_RCFILE)
  SERVER_RCFILE = File.join(SERVER_CONFDIR, 'tkorca_serverrc')
end

unless Object.const_defined?(:LOCAL_RCFILE)
  # LOCAL_RCFILE = '~/.tkorca_serverrc'
  LOCAL_RCFILE = nil
end


########################################################
#  get command-line options
########################################################
require 'server/opt_parse.rb'


########################################################
#  setup SERVER_PARAM
########################################################
SERVER_PARAM = {}

def setup_SERVER_PARAMS
  SERVER_PARAM.clear

  # hangup signal
  if OPTION['hangup_signal']
    SERVER_PARAM['hangup_signal'] = OPTION['hangup_signal']
  else
    SERVER_PARAM['hangup_signal'] = TkORCA_SIGHUP
  end
  unless SERVER_PARAM['hangup_signal']
    raise RuntimeError, "Fail to select proper singal for TkORCA_SIGHUP." + 
      " It is required for interactive control of application processes." + 
      " Please define TkORCA_SIGHUP on `server_base_defs' file" + 
      " or use `-S' (or `--hangup_signal') option."
  end

  #  server_rcfile
  if OPTION['serverrc']
    SERVER_PARAM['server_rcfile'] = File.expand_path(OPTION['serverrc'])
  elsif SERVER_RCFILE
    SERVER_PARAM['server_rcfile'] = File.expand_path(SERVER_RCFILE)
  else
    SERVER_PARAM['server_rcfile'] = nil
  end

  #  others
  if OPTION['tmp_dir']
    SERVER_PARAM['tmp_dir'] = File.expand_path(OPTION['tmp_dir'])
  elsif Object.const_defined?(:SERVER_TMPDIR)
    SERVER_PARAM['tmp_dir'] = File.expand_path(SERVER_TMPDIR)
  else
    SERVER_PARAM['tmp_dir'] = File.expand_path(Dir.tmpdir)
  end
end

setup_SERVER_PARAMS()


########################################################
#  get PID
########################################################
if Object.const_defined?(:SERVER_PID) && SERVER_PID
  SERVER_PARAM['server_pid'] = File.expand_path(SERVER_PID)
else
  SERVER_PARAM['server_pid'] = File.join(SERVER_PARAM['tmp_dir'], 'tkorca.pid')
end


########################################################
#  get logfilename
########################################################
if OPTION['logfile']
  logfile = File.expand_path(OPTION['logfile'])
elsif Object.const_defined?(:SERVER_LOGFILE) && SERVER_LOGFILE
  logfile = File.expand_path(SERVER_LOGFILE)
else
  logfile = File.join(SERVER_PARAM['tmp_dir'], 'tkorca.log')
end

########################################################
#  log rotate
########################################################
if File.exist?(logfile + ".#{LOG_BACKUP_MAX}")
  File.delete(logfile + ".#{LOG_BACKUP_MAX}")
end

(1..(LOG_BACKUP_MAX - 1)).to_a.reverse.each{|i|
  if File.exist?(src = (logfile + ".#{i}"))
    File.rename(src, logfile + ".#{i+1}")
  end
}

if File.exist?(logfile)
  File.rename(logfile, logfile + ".1")
end


########################################################
#  send HUP signal
########################################################
if File.exist?(SERVER_PARAM['server_pid'])
  pid = Integer(IO.readlines(SERVER_PARAM['server_pid'])[0])
  begin
    Process.kill(SERVER_PARAM['hangup_signal'], pid)
  rescue
  end
end
